@@ -80,13 +80,13 @@ prolog_replacements = {
     '{exc_description_wsl}': 'x86/tumbleweed/test-image-wsl',
     '{exc_description_docker}': 'x86/leap/test-image-docker',
     '{exc_description_enclave}': 'x86/rawhide/test-image-nitro-enclave',
-    '{exc_os_version}': '15.5',
-    '{exc_image_version}': '1.15.3',
-    '{exc_repo_leap}': 'obs://openSUSE:Leap:15.5/standard',
-    '{exc_repo_tumbleweed}': 'http://download.opensuse.org/tumbleweed/repo/oss',
+    '{exc_os_version}': '15.6',
+    '{exc_image_version}': '1.15.6',
+    '{exc_repo_leap}': 'https://download.opensuse.org/distribution/leap/15.6/repo/oss',
+    '{exc_repo_tumbleweed}': 'https://download.opensuse.org/tumbleweed/repo/oss',
     '{exc_repo_rawhide}': 'https://mirrors.fedoraproject.org/metalink?repo=rawhide&arch=x86_64',
     '{exc_kiwi_repo}':
-        'obs://Virtualization:Appliances:Builder/openSUSE_Leap_15.5',
+        'obs://Virtualization:Appliances:Builder/openSUSE_Leap_15.6',
     '{schema_version}': '8.0',
     '{kiwi}': 'KIWI NG',
     '{kiwi-product}': 'KIWI Next Generation (KIWI NG)',
@@ -142,7 +142,7 @@ copyright = f'2020-{datetime.now().year}, {author}'
 # built documents.
 #
 # The short X.Y version.
-version = '10.1.18'
+version = '10.2.24'
 # The full version, including alpha/beta/rc tags.
 release = version
 
