@@ -38,15 +38,7 @@ function initGlobalDevices {
     elif [[ "${root_cmdline}" =~ "overlay:aoe=" ]];then
         read_only_partition="/dev/etherd/$(echo "${root_cmdline}"|cut -f2 -d=)"
     else
-        write_partition="$1"
-        root_disk=$(
-            lsblk -p -n -r -s -o NAME,TYPE "${write_partition}" \
-                | grep disk | cut -f1 -d ' '
-        )
-        read_only_partition=$(
-            lsblk -p -r --fs -o NAME,FSTYPE "${root_disk}" \
-                | grep squashfs | head -n 1 | cut -f1 -d ' '
-        )
+        read_only_partition="$1"
     fi
 }
 
@@ -80,6 +72,22 @@ function preparePersistentOverlay {
     mkdir -m 0755 -p "${overlay_mount_point}/work"
 }
 
+function preparePersistentOverlayFromFstab {
+    local overlay_base
+    overlay_base=$(getOverlayBaseDirectory)
+    local root_mount_point="${overlay_base}/rootfsbase"
+    local overlay_mount_point="${overlay_base}/overlayfs"
+    while read -r overlay;do
+    if [[ "${overlay}" =~ ^overlay ]];then
+        for i in $(echo "${overlay}" | tr , " ");do
+            if [[ "$i" =~ ^upperdir|^workdir ]];then
+                mkdir -p "$(echo "$i" | cut -f2 -d=)"
+            fi
+        done
+    fi
+    done < "${root_mount_point}/etc/fstab"
+}
+
 #======================================
 # perform root access preparation
 #--------------------------------------
@@ -107,11 +115,11 @@ loadKernelModules
 # mount readonly root filesystem
 mountReadOnlyRootImage
 
-# prepare overlay for generated systemd OverlayOS_rootfs service
-if [ -z "${write_partition}" ] || getargbool 0 rd.root.overlay.readonly; then
+if [ -z "${write_partition}" ] || getargbool 0 rd.root.overlay.temporary; then
     prepareTemporaryOverlay
 else
     preparePersistentOverlay
+    preparePersistentOverlayFromFstab
 fi
 
 need_shutdown
