@@ -60,6 +60,7 @@ class BootLoaderConfigBase(ABC):
         self.proc_mount = None
         self.sys_mount = None
         self.tmp_mount = None
+        self.etc_kernel_mount = None
 
         self.root_filesystem_is_overlay = xml_state.build_type.get_overlayroot()
         self.post_init(custom_args)
@@ -156,7 +157,9 @@ class BootLoaderConfigBase(ABC):
         """
 
     @abstractmethod
-    def setup_disk_boot_images(self, boot_uuid, lookup_path=None):
+    def setup_disk_boot_images(
+        self, boot_uuid, efi_uuid=None, lookup_path=None
+    ):
         """
         Create bootloader images for disk boot
 
@@ -165,6 +168,7 @@ class BootLoaderConfigBase(ABC):
         path on a filesystem.
 
         :param string boot_uuid: boot device UUID
+        :param string efi_uuid: EFI device UUID
         :param string lookup_path: custom module lookup path
 
         Implementation in specialized bootloader class required
@@ -555,16 +559,24 @@ class BootLoaderConfigBase(ABC):
 
         if self.root_filesystem_is_overlay:
             # In case of an overlay root system all parts of the rootfs
-            # are read-only by squashfs except for the extra boot partition.
-            # However tools like grub's mkconfig creates temporary files
-            # at call time and therefore /tmp needs to be writable during
-            # the call time of the tools
+            # are read-only. However tools like grub's mkconfig creates
+            # temporary files at call time and therefore /tmp needs to
+            # be writable during the call time of the tools
             self.tmp_mount = MountManager(
                 device='/tmp',
                 mountpoint=self.root_mount.mountpoint + '/tmp'
             )
             self.tmp_mount.bind_mount()
 
+            # There are also tools that writes to /etc/kernel, e.g
+            # systemd-boot. If it exists we map it to the ESP
+            etc_kernel = f'{self.root_mount.mountpoint}/etc/kernel'
+            if os.path.exists(etc_kernel):
+                self.etc_kernel_mount = MountManager(
+                    device=efi_device, mountpoint=etc_kernel
+                )
+                self.etc_kernel_mount.mount()
+
         self.device_mount = MountManager(
             device='/dev',
             mountpoint=self.root_mount.mountpoint + '/dev'
@@ -600,6 +612,8 @@ class BootLoaderConfigBase(ABC):
                 self.efi_mount.umount()
             if self.tmp_mount:
                 self.tmp_mount.umount()
+            if self.etc_kernel_mount:
+                self.etc_kernel_mount.umount()
             if self.boot_mount:
                 self.boot_mount.umount()
             if self.root_mount:
@@ -626,13 +640,24 @@ class BootLoaderConfigBase(ABC):
                 return root_search.group(1)
         if boot_device:
             if self.xml_state.build_type.get_overlayroot():
-                # In case of an overlay setup the root partition is a squashfs
-                # In this case the root location can only be specified by the
-                # partition uuid because squashfs itself doesn't have one
-                root_location = self._get_location(boot_device, 'by-partuuid')
-                return 'root=overlay:{0}={1}'.format(
-                    root_location['type'], root_location['name']
-                )
+                # In case of an overlay setup the root partition is read-only
+                # In this case the root location will be specified by the
+                # partition uuid because not all read-only filesystems have one.
+                # Exception to this is if the overlay is also encrypted
+                # Exception to this is if the overlay is on verity
+                verity = self.xml_state.build_type.get_verity_blocks()
+                luks = self.xml_state.get_luks_credentials()
+                if luks is not None:
+                    return 'root=overlay:MAPPER=luks'
+                elif verity:
+                    return 'root=overlay:MAPPER=verityroot'
+                else:
+                    root_location = self._get_location(
+                        boot_device, 'by-partuuid'
+                    )
+                    return 'root=overlay:{0}={1}'.format(
+                        root_location['type'], root_location['name']
+                    )
             else:
                 root_location = self._get_location(boot_device)
                 return 'root={0}={1}'.format(
