@@ -18,6 +18,7 @@
 from string import Template
 from textwrap import dedent
 import re
+import sys
 import os
 import logging
 import glob
@@ -35,6 +36,7 @@ from kiwi.path import Path
 from kiwi.system.identifier import SystemIdentifier
 from kiwi.utils.sync import DataSync
 from kiwi.utils.sysconfig import SysConfig
+from kiwi.utils.temporary import Temporary
 
 from kiwi.exceptions import (
     KiwiTemplateError,
@@ -51,6 +53,13 @@ import kiwi.defaults as defaults
 log = logging.getLogger('kiwi')
 
 
+def func_name():
+    """
+    helper method to return name of caller
+    """
+    return sys._getframe(1).f_code.co_name
+
+
 class BootLoaderConfigGrub2(BootLoaderConfigBase):
     """
     **grub2 bootloader configuration.**
@@ -66,6 +75,7 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
 
                 {'grub_directory_name': 'grub|grub2'}
         """
+        self.host_key_certificates = self.xml_state.get_host_key_certificates()
         self.efi_csm = True if self.xml_state.build_type.get_eficsm() is None \
             else self.xml_state.build_type.get_eficsm()
         self.custom_args = custom_args
@@ -74,6 +84,9 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
         if arch == 'x86_64':
             # grub2 support for bios and efi systems
             self.arch = arch
+        elif arch == 'loongarch64':
+            # grub2 support for efi systems
+            self.arch = arch
         elif arch.startswith('ppc64'):
             # grub2 support for ofw and opal systems
             self.arch = arch
@@ -311,16 +324,28 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 ['bash', '-c', ' '.join(bash_command)], raise_on_error=False
             )
 
+        # Create UKI image if requested
+        dracut_setup = self.xml_state.get_dracut_config('setup')
+        if dracut_setup.uefi:
+            boot_image = boot_options.get('boot_image')
+            if boot_image:
+                efi_uki_path = \
+                    f'{self.root_mount.mountpoint}/boot/efi/EFI/Linux/'
+                Path.create(efi_uki_path)
+                uki_image = boot_image.create_uki(self.cmdline)
+                Command.run(
+                    ['mv', uki_image, efi_uki_path]
+                )
+
         # Patch the written grub config file to actually work:
         # Unfortunately the grub tooling has several bugs and issues
         # which prevents it to work in image build environments.
         # More details can be found in the individual methods.
         # One fine day the following fix methods can hopefully
         # be deleted...
-        if self.xml_state.build_type.get_overlayroot_write_partition() is not False:
-            self._fix_grub_root_device_reference(config_file, boot_options)
-            self._fix_grub_loader_entries_boot_cmdline()
-            self._fix_grub_loader_entries_linux_and_initrd_paths()
+        self._fix_grub_root_device_reference(config_file, boot_options)
+        self._fix_grub_loader_entries_boot_cmdline()
+        self._fix_grub_loader_entries_linux_and_initrd_paths()
 
         if self.firmware.efi_mode() and self.early_boot_script_efi:
             self._copy_grub_config_to_efi_path(
@@ -470,7 +495,9 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 self.boot_dir, self.early_boot_script_efi, 'iso'
             )
 
-    def setup_install_boot_images(self, mbrid: SystemIdentifier, lookup_path: str = None) -> None:
+    def setup_install_boot_images(
+        self, mbrid: SystemIdentifier, lookup_path: str = None
+    ) -> None:
         """
         Create/Provide grub2 boot images and metadata
 
@@ -499,15 +526,17 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
 
         self._copy_theme_data_to_boot_directory(lookup_path, 'iso')
 
-        if self._supports_bios_modules():
+        if self._supports_platform_modules():
             self._copy_bios_modules_to_boot_directory(lookup_path)
-            self._setup_bios_image(mbrid=mbrid, lookup_path=lookup_path)
+            self._setup_platform_image(mbrid=mbrid, lookup_path=lookup_path)
 
         if self.firmware.efi_mode():
             self._setup_EFI_path(lookup_path)
 
         if self.firmware.efi_mode() == 'efi':
-            self._setup_efi_image(mbrid=mbrid, lookup_path=lookup_path, target_type='iso')
+            self._setup_efi_image(
+                mbrid=mbrid, lookup_path=lookup_path, target_type='iso'
+            )
             self._copy_efi_modules_to_boot_directory(lookup_path)
         elif self.firmware.efi_mode() == 'uefi':
             self._setup_secure_boot_efi_image(
@@ -530,7 +559,9 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
         """
         self.setup_install_boot_images(mbrid, lookup_path)
 
-    def setup_disk_boot_images(self, boot_uuid, lookup_path=None):
+    def setup_disk_boot_images(
+        self, boot_uuid, efi_uuid=None, lookup_path=None
+    ):
         """
         Create/Provide grub2 boot images and metadata
 
@@ -548,17 +579,18 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
 
         self._copy_theme_data_to_boot_directory(lookup_path, 'disk')
 
-        if not self.xen_guest and self._supports_bios_modules():
+        if not self.xen_guest and self._supports_platform_modules():
             self._copy_bios_modules_to_boot_directory(lookup_path)
 
         if self.firmware.efi_mode() == 'efi':
-            self._setup_efi_image(uuid=boot_uuid, lookup_path=lookup_path)
+            self._setup_efi_image(
+                uuid=boot_uuid, efi_uuid=efi_uuid, lookup_path=lookup_path
+            )
             self._copy_efi_modules_to_boot_directory(lookup_path)
         elif self.firmware.efi_mode() == 'uefi':
-            self._copy_efi_modules_to_boot_directory(lookup_path)
             if not self._get_shim_install():
                 self._setup_secure_boot_efi_image(
-                    lookup_path=lookup_path, uuid=boot_uuid
+                    lookup_path=lookup_path, uuid=boot_uuid, efi_uuid=efi_uuid
                 )
 
         if self.xen_guest:
@@ -609,11 +641,11 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                     config_file, grub_config_file_for_efi_boot
                 )
             )
-            shutil.copy(
+            shutil.copy2(
                 config_file, grub_config_file_for_efi_boot
             )
 
-    def _supports_bios_modules(self):
+    def _supports_platform_modules(self):
         if self.efi_csm and (
             self.arch == 'ix86' or self.arch == 'x86_64' or Defaults.is_ppc64_arch(self.arch)
         ):
@@ -622,8 +654,10 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
 
     def _setup_sysconfig_bootloader(self):
         """
-        Create or update etc/sysconfig/bootloader by the following
-        parameters required according to the grub2 bootloader setup
+        Create or update etc/sysconfig/bootloader for the following
+        parameters. Please note this file is consumed by perl-bootloader
+        only and marked for deletion with the next major release of
+        kiwi because not a mainline grub config file.
 
         * LOADER_TYPE
         * LOADER_LOCATION
@@ -638,6 +672,8 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             'LOADER_LOCATION':
                 'none' if self.firmware.efi_mode() else 'mbr'
         }
+        if self.bls:
+            sysconfig_bootloader_entries['LOADER_TYPE'] = 'grub2-bls'
         if '--set-trusted-boot' in self.config_options:
             sysconfig_bootloader_entries['TRUSTED_BOOT'] = 'yes'
         if self.firmware.efi_mode() == 'uefi':
@@ -684,10 +720,10 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
         if target_type and os.path.exists(zipl2grub_config_file):
             if os.path.exists(zipl2grub_config_file_orig):
                 # reset the original template file first
-                shutil.copy(zipl2grub_config_file_orig, zipl2grub_config_file)
+                shutil.copy2(zipl2grub_config_file_orig, zipl2grub_config_file)
             else:
                 # no copy of the original template, create it
-                shutil.copy(zipl2grub_config_file, zipl2grub_config_file_orig)
+                shutil.copy2(zipl2grub_config_file, zipl2grub_config_file_orig)
             with open(zipl2grub_config_file) as zipl_config_file:
                 zipl_config = zipl_config_file.read()
                 zipl_config = re.sub(
@@ -698,7 +734,7 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                     '    targetblocksize = {2}\n'
                     '    targetoffset = {3}\n'
                     '    {4}'.format(
-                        target_type,
+                        target_type if target_type != 'GPT' else 'SCSI',
                         self.custom_args['targetbase'],
                         target_blocksize or 512,
                         self._get_partition_start(
@@ -799,7 +835,7 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             )
             if os.path.exists(os.sep.join([self.root_dir, theme_background])):
                 grub_default_entries['GRUB_BACKGROUND'] = theme_background
-        if self.xml_state.build_type.get_btrfs_root_is_snapshot():
+        if self.xml_state.build_type.get_btrfs_root_is_snapper_snapshot():
             grub_default_entries['SUSE_BTRFS_SNAPSHOT_BOOTING'] = 'true'
         if self.custom_args.get('crypto_disk'):
             grub_default_entries['GRUB_ENABLE_CRYPTODISK'] = 'y'
@@ -813,6 +849,31 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
         if self.bls and enable_blscfg_implemented.returncode == 0:
             grub_default_entries['GRUB_ENABLE_BLSCFG'] = 'true'
 
+        if self.arch.startswith('s390') and self.host_key_certificates:
+            # At the moment no generic enabler for secure execution
+            # on s390 exists in grub upstream. Thus the following
+            # variable setup is distribution specific:
+            # Enable Secure Execution: SUSE specific variables
+            host_key_list = []
+            signing_key_list = []
+            revocation_key_list = []
+            grub_default_entries['SUSE_S390_SE_ENABLE'] = 'true'
+            grub_default_entries['SUSE_S390_SE_CA_CERT'] = \
+                self.host_key_certificates[0]['hkd_ca_cert']
+            for host_key_certificate in self.host_key_certificates:
+                signing_key_list.append(host_key_certificate['hkd_sign_cert'])
+                host_key_list += host_key_certificate['hkd_cert']
+                if host_key_certificate.get('hkd_revocation_list'):
+                    revocation_key_list += \
+                        host_key_certificate['hkd_revocation_list']
+            grub_default_entries['SUSE_S390_SE_HOST_KEY'] = \
+                ','.join(host_key_list)
+            grub_default_entries['SUSE_S390_SE_HOST_KEY_SIGNING_KEY'] = \
+                ','.join(signing_key_list)
+            if revocation_key_list:
+                grub_default_entries['SUSE_S390_SE_REVOCATION_LIST'] = \
+                    ','.join(revocation_key_list)
+
         if grub_default_entries:
             log.info('Writing grub2 defaults file')
             grub_default_location = ''.join([self.root_dir, '/etc/default/'])
@@ -835,7 +896,8 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 grub_default.write()
 
     def _setup_secure_boot_efi_image(
-        self, lookup_path, uuid=None, mbrid=None, target_type='disk'
+        self, lookup_path, uuid=None, efi_uuid=None,
+        mbrid=None, target_type='disk'
     ):
         """
         Provide the shim loader and the shim signed grub2 loader
@@ -906,10 +968,11 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 Command.run(
                     ['cp', grub_image.filename, target_efi_image_name]
                 )
-        self._create_efi_config_search(uuid, mbrid)
+        self._create_efi_config_search(uuid, efi_uuid, mbrid)
 
     def _setup_efi_image(
-        self, uuid=None, mbrid=None, lookup_path=None, target_type='disk'
+        self, uuid=None, efi_uuid=None, mbrid=None,
+        lookup_path=None, target_type='disk'
     ):
         """
         Provide the unsigned grub2 efi image in the required boot path
@@ -926,11 +989,11 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             Command.run(
                 ['cp', grub_image, self._get_efi_image_name()]
             )
-            self._create_efi_config_search(uuid, mbrid)
+            self._create_efi_config_search(uuid, efi_uuid, mbrid)
         else:
             log.info('--> Creating unsigned efi image')
             self._create_efi_image(
-                uuid, mbrid, lookup_path
+                uuid, efi_uuid, mbrid, lookup_path
             )
 
     def _setup_chrp_config(self, mbrid):
@@ -949,26 +1012,29 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             <chrp-boot>
             <description>{os_name}</description>
             <os-name>{os_name}</os-name>
-            <boot-script>boot &device;:1,\\boot\\grub2\\powerpc-ieee1275\\grub.elf</boot-script>
+            <boot-script>boot &device;:1,\\boot\\grub2\\powerpc-ieee1275\\{chrp_loader}</boot-script>
             </chrp-boot>
         ''').strip() + os.linesep
         with open(chrp_bootinfo_file, 'w') as chrp_bootinfo:
             chrp_bootinfo.write(
-                chrp_config.format(os_name=self.get_menu_entry_install_title())
+                chrp_config.format(
+                    os_name=self.get_menu_entry_install_title(),
+                    chrp_loader=Defaults.get_grub_chrp_loader(self.boot_dir)
+                )
             )
 
-    def _setup_bios_image(self, mbrid, lookup_path=None):
+    def _setup_platform_image(self, mbrid, lookup_path=None):
         """
         Provide bios grub image
         """
         if not lookup_path:
             lookup_path = self.boot_dir
-        grub_image = Defaults.get_grub_bios_core_loader(lookup_path)
+        grub_image = Defaults.get_grub_platform_core_loader(lookup_path)
         if grub_image:
             log.info(f'--> Using prebuilt bios image: {grub_image}')
         else:
             log.info('--> Creating bios image')
-            self._create_bios_image(
+            self._create_platform_image(
                 mbrid, lookup_path
             )
         if Defaults.is_ppc64_arch(Defaults.get_platform_name()):
@@ -977,12 +1043,14 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
         bash_command = ' '.join(
             [
                 'cat',
-                self._get_bios_modules_path(lookup_path) + '/cdboot.img',
-                grub_image or self._get_bios_image_name(lookup_path),
+                self._get_grub_platform_modules_path(
+                    lookup_path
+                ) + '/cdboot.img',
+                grub_image or self._get_grub_platform_image_name(lookup_path),
                 '>',
                 os.sep.join(
                     [
-                        self._get_bios_modules_path(lookup_path),
+                        self._get_grub_platform_modules_path(lookup_path),
                         Defaults.get_iso_grub_loader()
                     ]
                 )
@@ -1001,6 +1069,12 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             self.xml_state.build_type
                 .get_efifatimagesize() or defaults.EFI_FAT_IMAGE_SIZE
         )
+        efi_folder = Temporary(prefix='efi_folder').new_dir()
+        efi_data = DataSync(self.boot_dir + '/EFI/', efi_folder.name)
+        efi_data.sync_data(
+            options=Defaults.get_sync_options(),
+            exclude=Defaults.get_exclude_list_from_custom_exclude_files_for_efifatimage(self.root_dir)
+        )
         Command.run(
             ['qemu-img', 'create', path, f'{fat_image_mbsize}M']
         )
@@ -1010,16 +1084,23 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
         Command.run(
             [
                 'mcopy', '-Do', '-s', '-i', path,
-                self.boot_dir + '/EFI', '::'
+                efi_folder.name, '::EFI'
             ]
         )
 
-    def _create_efi_image(self, uuid=None, mbrid=None, lookup_path=None):
+    def _create_efi_image(
+        self, uuid=None, efi_uuid=None, mbrid=None, lookup_path=None
+    ):
+        dracut_setup = self.xml_state.get_dracut_config('setup')
         early_boot_script = os.path.normpath(
             os.sep.join([self.efi_boot_path, 'earlyboot.cfg'])
         )
         self.early_boot_script_efi = early_boot_script
-        if uuid:
+        if dracut_setup.uefi and efi_uuid:
+            self._create_early_boot_script_for_uki_chainload(
+                early_boot_script, efi_uuid
+            )
+        elif uuid:
             self._create_early_boot_script_for_uuid_search(
                 early_boot_script, uuid
             )
@@ -1043,7 +1124,7 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             log.debug(
                 f'Copy earlyboot to media path: {early_boot_script_on_media}'
             )
-            shutil.copy(
+            shutil.copy2(
                 early_boot_script, early_boot_script_on_media
             )
         mkimage_call = Command.run(
@@ -1073,12 +1154,17 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 f'Copy grub image to media path: {efi_image_media_file}'
             )
             Path.create(os.path.dirname(efi_image_media_file))
-            shutil.copy(efi_image_root_file, efi_image_media_file)
+            shutil.copy2(efi_image_root_file, efi_image_media_file)
 
-    def _create_efi_config_search(self, uuid=None, mbrid=None):
+    def _create_efi_config_search(self, uuid=None, efi_uuid=None, mbrid=None):
+        dracut_setup = self.xml_state.get_dracut_config('setup')
         efi_boot_config = self.efi_boot_path + '/grub.cfg'
         self.early_boot_script_efi = efi_boot_config
-        if uuid:
+        if dracut_setup.uefi and efi_uuid:
+            self._create_early_boot_script_for_uki_chainload(
+                efi_boot_config, efi_uuid
+            )
+        elif uuid:
             self._create_early_boot_script_for_uuid_search(
                 efi_boot_config, uuid
             )
@@ -1087,7 +1173,7 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 efi_boot_config, mbrid
             )
 
-    def _create_bios_image(self, mbrid, lookup_path=None):
+    def _create_platform_image(self, mbrid, lookup_path=None):
         early_boot_script = os.path.normpath(
             os.sep.join([self._get_grub2_boot_path(), 'earlyboot.cfg'])
         )
@@ -1104,7 +1190,7 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             Path.create(
                 os.path.dirname(early_boot_script_on_media)
             )
-            shutil.copy(
+            shutil.copy2(
                 early_boot_script, early_boot_script_on_media
             )
         mkimage_call = Command.run(
@@ -1113,25 +1199,25 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 os.path.basename(
                     self._get_grub2_mkimage_tool()
                 ) or 'grub2-mkimage',
-                '-O', Defaults.get_bios_module_directory_name(),
-                '-o', self._get_bios_image_name(self.root_dir).replace(
+                '-O', Defaults.get_grub_platform_module_directory_name(),
+                '-o', self._get_grub_platform_image_name(self.root_dir).replace(
                     self.root_dir, ''
                 ),
                 '-c', early_boot_script.replace(self.boot_dir, ''),
                 '-p', os.sep.join(
                     [self.get_boot_path(), self.boot_directory_name]
                 ),
-                '-d', self._get_bios_modules_path(self.root_dir).replace(
-                    self.root_dir, ''
-                )
-            ] + Defaults.get_grub_bios_modules(multiboot=self.xen_guest)
+                '-d', self._get_grub_platform_modules_path(
+                    self.root_dir
+                ).replace(self.root_dir, '')
+            ] + Defaults.get_grub_platform_modules(multiboot=self.xen_guest)
         )
         log.debug(mkimage_call.output)
 
         # Copy generated EFI image to the media directory if this
         # is different from the system root directory, e.g the case
         # for live image builds or when using a custom kiwi initrd
-        bios_image_root_file = self._get_bios_image_name(self.root_dir)
+        bios_image_root_file = self._get_grub_platform_image_name(self.root_dir)
         bios_image_media_file = bios_image_root_file.replace(
             self.root_dir, lookup_path
         )
@@ -1140,7 +1226,17 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 f'Copy grub image to media path: {bios_image_media_file}'
             )
             Path.create(os.path.dirname(bios_image_media_file))
-            shutil.copy(bios_image_root_file, bios_image_media_file)
+            shutil.copy2(bios_image_root_file, bios_image_media_file)
+
+    def _create_early_boot_script_for_uki_chainload(self, filename, uuid):
+        with open(filename, 'w') as early_boot:
+            early_boot.write(
+                f'search --no-floppy --set=root --fs-uuid {uuid}{os.linesep}'
+            )
+            early_boot.write(
+                f'chainloader ($root)/EFI/Linux/kiwi.efi{os.linesep}'
+            )
+            early_boot.write(f'boot{os.linesep}')
 
     def _create_early_boot_script_for_uuid_search(self, filename, uuid):
         with open(filename, 'w') as early_boot:
@@ -1203,21 +1299,25 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                 )
             )
 
-    def _get_grub2_mkimage_tool(self):
+    def _get_grub2_mkimage_tool(self) -> str:
+        grub_mkimage_file_path = ''
         for grub_mkimage_tool in ['grub2-mkimage', 'grub-mkimage']:
             grub_mkimage_file_path = Path.which(
                 grub_mkimage_tool, root_dir=self.root_dir
-            )
+            ) or ''
             if grub_mkimage_file_path:
-                return grub_mkimage_file_path
+                break
+        return grub_mkimage_file_path
 
-    def _get_grub2_mkconfig_tool(self):
+    def _get_grub2_mkconfig_tool(self) -> str:
+        grub_mkconfig_file_path = ''
         for grub_mkconfig_tool in ['grub2-mkconfig', 'grub-mkconfig']:
             grub_mkconfig_file_path = Path.which(
                 grub_mkconfig_tool, root_dir=self.root_dir
-            )
+            ) or ''
             if grub_mkconfig_file_path:
-                return grub_mkconfig_file_path
+                break
+        return grub_mkconfig_file_path
 
     def _get_grub2_boot_path(self):
         return self.boot_dir + '/boot/' + self.boot_directory_name
@@ -1230,11 +1330,11 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             ]
         )
 
-    def _get_bios_image_name(self, lookup_path):
+    def _get_grub_platform_image_name(self, lookup_path):
         return os.sep.join(
             [
-                self._get_bios_modules_path(lookup_path),
-                Defaults.get_bios_image_name()
+                self._get_grub_platform_modules_path(lookup_path),
+                Defaults.get_grub_platform_image_name()
             ]
         )
 
@@ -1244,8 +1344,10 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
             lookup_path
         )
 
-    def _get_bios_modules_path(self, lookup_path=None):
-        return self._get_module_path(Defaults.get_bios_module_directory_name(), lookup_path)
+    def _get_grub_platform_modules_path(self, lookup_path=None):
+        return self._get_module_path(
+            Defaults.get_grub_platform_module_directory_name(), lookup_path
+        )
 
     def _get_xen_modules_path(self, lookup_path=None):
         return self._get_module_path(
@@ -1389,7 +1491,7 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
 
     def _copy_bios_modules_to_boot_directory(self, lookup_path):
         self._copy_modules_to_boot_directory_from(
-            self._get_bios_modules_path(lookup_path)
+            self._get_grub_platform_modules_path(lookup_path)
         )
 
     def _copy_xen_modules_to_boot_directory(self, lookup_path):
@@ -1441,8 +1543,12 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                     grub_config
                 )
 
-            with open(config_file, 'w') as grub_config_file:
-                grub_config_file.write(grub_config)
+            try:
+                with open(config_file, 'w') as grub_config_file:
+                    grub_config_file.write(grub_config)
+            except OSError as issue:
+                log.warning(f'{func_name()} skipped: {issue}')
+                return
 
             if self.firmware.efi_mode():
                 vendor_grubenv_file = \
@@ -1488,8 +1594,12 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                         r'options {0}'.format(self.cmdline),
                         menu_entry
                     )
-                with open(menu_entry_file, 'w') as grub_menu_entry_file:
-                    grub_menu_entry_file.write(menu_entry)
+                try:
+                    with open(menu_entry_file, 'w') as grub_menu_entry_file:
+                        grub_menu_entry_file.write(menu_entry)
+                except OSError as issue:
+                    log.warning(f'{func_name()} skipped: {issue}')
+                    return
 
     def _fix_grub_loader_entries_linux_and_initrd_paths(self):
         # For the same reasons encoded in _fix_grub_loader_entries_boot_cmdline
@@ -1521,8 +1631,11 @@ class BootLoaderConfigGrub2(BootLoaderConfigBase):
                     log.debug(f'Updated loader entry: {config_path}')
 
             menu_entry = os.linesep.join(menu_entry)
-            with open(menu_entry_file, 'w') as grub_menu_entry_file:
-                grub_menu_entry_file.write(menu_entry)
+            try:
+                with open(menu_entry_file, 'w') as grub_menu_entry_file:
+                    grub_menu_entry_file.write(menu_entry)
+            except OSError as issue:
+                log.warning(f'{func_name()} skipped: {issue}')
 
     def _get_partition_start(self, disk_device):
         if self.target_table_type == 'dasd':
