@@ -30,6 +30,8 @@ else:  # pragma: no cover
 # project
 import kiwi.defaults as defaults
 
+from kiwi.path import Path
+from kiwi.mount_manager import MountManager
 from kiwi.utils.veritysetup import VeritySetup
 from kiwi.utils.temporary import Temporary
 from kiwi.system.mount import ImageSystem
@@ -41,6 +43,7 @@ from kiwi.bootloader.config.base import BootLoaderConfigBase
 from kiwi.bootloader.install import BootLoaderInstall
 from kiwi.system.identifier import SystemIdentifier
 from kiwi.boot.image import BootImage
+from kiwi.boot.image.base import BootImageBase
 from kiwi.storage.setup import DiskSetup
 from kiwi.storage.loop_device import LoopDevice
 from kiwi.storage.clone_device import CloneDevice
@@ -54,7 +57,6 @@ from kiwi.storage.integrity_device import (
 )
 from kiwi.storage.device_provider import DeviceProvider
 from kiwi.filesystem import FileSystem
-from kiwi.filesystem.squashfs import FileSystemSquashFs
 from kiwi.volume_manager import VolumeManager
 from kiwi.volume_manager.base import VolumeManagerBase
 from kiwi.command import Command
@@ -116,6 +118,7 @@ class DiskBuilder:
         self.root_dir = root_dir
         self.target_dir = target_dir
         self.xml_state = xml_state
+        self.cmdline = xml_state.build_type.get_kernelcmdline() or ''
         self.spare_part_mbsize = xml_state.get_build_type_spare_part_size()
         self.spare_part_fs = xml_state.build_type.get_spare_part_fs()
         self.spare_part_is_last = xml_state.build_type.get_spare_part_is_last()
@@ -123,10 +126,13 @@ class DiskBuilder:
             xml_state.build_type.get_spare_part_mountpoint()
         self.persistency_type = xml_state.build_type.get_devicepersistency()
         self.root_filesystem_is_overlay = xml_state.build_type.get_overlayroot()
+        self.root_filesystem_read_only_type = \
+            xml_state.build_type.get_overlayroot_readonly_filesystem()
         self.root_filesystem_has_write_partition = \
             xml_state.build_type.get_overlayroot_write_partition()
         self.root_filesystem_read_only_partsize = \
             xml_state.build_type.get_overlayroot_readonly_partsize()
+        self.veritysetup = None
         self.root_filesystem_verity_blocks = \
             xml_state.build_type.get_verity_blocks()
         self.root_filesystem_embed_verity_metadata = \
@@ -149,6 +155,9 @@ class DiskBuilder:
         self.volumes = xml_state.get_volumes()
         self.custom_partitions = xml_state.get_partitions()
         self.volume_group_name = xml_state.get_volume_group_name()
+        self.dracut_setup = xml_state.get_dracut_config('setup')
+        self.dracut_add_modules = xml_state.get_dracut_config('add').modules
+        self.dracut_omit_modules = xml_state.get_dracut_config('omit').modules
         self.mdraid = xml_state.build_type.get_mdraid()
         self.hybrid_mbr = xml_state.build_type.get_gpt_hybrid_mbr()
         self.force_mbr = xml_state.build_type.get_force_mbr()
@@ -226,8 +235,8 @@ class DiskBuilder:
                 self.custom_root_creation_args,
             'root_label':
                 self.disk_setup.get_root_label(),
-            'root_is_snapshot':
-                self.xml_state.build_type.get_btrfs_root_is_snapshot(),
+            'root_is_snapper_snapshot':
+                self.xml_state.build_type.get_btrfs_root_is_snapper_snapshot(),
             'root_is_readonly_snapshot':
                 self.xml_state.build_type.get_btrfs_root_is_readonly_snapshot(),
             'root_is_subvolume':
@@ -667,7 +676,7 @@ class DiskBuilder:
         #    LUKS pool without asking
         #
         luks_need_keyfile = \
-            True if self.boot_is_crypto or self.luks == '' else False
+            True if self.boot_is_crypto or self.luks == '' or self.luks == 'random' else False
         luks_root.create_crypto_luks(
             passphrase=self.luks or '',
             osname=self.luks_os,
@@ -782,11 +791,18 @@ class DiskBuilder:
 
         self._write_crypttab_to_system_image(luks_root)
 
+        if 'rd.kiwi.oem.luks.reencrypt' in self.cmdline:
+            self._write_luks_header_checksum_to_boot_image(luks_root)
+
         self._write_integritytab_to_system_image(integrity_root)
 
         self._write_generic_fstab_to_system_image(device_map, system)
 
         if self.initrd_system == 'dracut':
+            for module in self.dracut_add_modules:
+                self.boot_image.include_module(module)
+            for module in self.dracut_omit_modules:
+                self.boot_image.omit_module(module)
             if self.root_filesystem_is_multipath is False:
                 self.boot_image.omit_module('multipath')
             if self.root_filesystem_is_overlay:
@@ -798,7 +814,7 @@ class DiskBuilder:
                 self.boot_image.include_module('kiwi-repart')
 
         # create initrd
-        if self.boot_image.has_initrd_support():
+        if self.boot_image.has_initrd_support() and not self.dracut_setup.uefi:
             self.boot_image.create_initrd(self.mbrid)
 
         # create second stage metadata to system image
@@ -835,7 +851,7 @@ class DiskBuilder:
                 disk_system.call_pre_disk_script()
 
             # syncing system data to disk image
-            self._sync_system_to_image(
+            system = self._sync_system_to_image(
                 stack,
                 device_map,
                 system,
@@ -846,26 +862,77 @@ class DiskBuilder:
                 integrity_root
             )
 
-            # run post sync script hook
-            if system and self.system_setup.script_exists(
-                defaults.POST_DISK_SYNC_SCRIPT
-            ):
+            # run post sync actions...
+            if self.veritysetup:
+                self._write_veritytab_to_boot_image(
+                    device_map, self.veritysetup
+                )
+
+            if system:
                 with ImageSystem(
                     device_map, self.root_dir,
-                    system.get_volumes() if self.volume_manager_name else {},
+                    system if self.volume_manager_name else None,
                     self.custom_partitions if self.custom_partitions else {}
                 ) as image_system:
                     image_system.mount()
+                    root = image_system.mountpoint()
+
+                    # rebuild initrd with veritytab included
+                    if self.veritysetup and \
+                       self.boot_image.has_initrd_support() and not \
+                       self.dracut_setup.uefi:
+                        self.boot_image.create_initrd(self.mbrid)
+                        self._copy_first_boot_files_to_system_image()
+                        boot_names = self.boot_image.get_boot_names()
+                        if os.access(f'{root}/boot/', os.W_OK):
+                            Command.run(
+                                [
+                                    'cp',
+                                    f'{self.root_dir}/boot/{boot_names.initrd_name}',
+                                    f'{root}/boot/{boot_names.initrd_name}'
+                                ]
+                            )
+
                     disk_system = SystemSetup(
                         self.xml_state, image_system.mountpoint()
                     )
-                    disk_system.call_disk_script()
+                    if self.system_setup.script_exists(
+                        defaults.POST_DISK_SYNC_SCRIPT
+                    ):
+                        # run post sync script hook and security context
+                        disk_system.call_disk_script()
+                    elif not self.root_filesystem_is_overlay:
+                        # setup security context
+                        disk_system.setup_selinux_file_contexts()
 
-            # install boot loader
-            if self.bootloader != 'custom':
-                self._install_bootloader(
-                    device_map, disk, system, bootloader_config
-                )
+                    # install boot loader
+                    if self.bootloader != 'custom':
+                        self._install_bootloader(
+                            device_map, disk, system, bootloader_config,
+                            self.boot_image
+                        )
+
+                    # initrd update on the ESP for systemd for read-only boot/
+                    if self.veritysetup and \
+                       self.boot_image.has_initrd_support() and not \
+                       self.dracut_setup.uefi and not \
+                       os.access(f'{root}/boot/', os.W_OK) and \
+                       self.bootloader == 'systemd_boot':
+                        Command.run(
+                            [
+                                'cp',
+                                f'{self.root_dir}/boot/{boot_names.initrd_name}',
+                                f'{root}/boot/efi/os/{boot_names.initrd_name}'
+                            ]
+                        )
+
+                    # cleanup build metadata
+                    if os.access(f'{root}/', os.W_OK):
+                        disk_system.cleanup()
+
+                    # set root filesystem properties
+                    if system:
+                        self._setup_property_root_is_readonly_snapshot(system)
 
             # call edit_boot_install script
             boot_device = device_map['root']
@@ -875,10 +942,6 @@ class DiskBuilder:
                 self.diskname, boot_device.get_device()
             )
 
-            # set root filesystem properties
-            if system:
-                self._setup_property_root_is_readonly_snapshot(system)
-
     def _install_image_requested(self) -> bool:
         return bool(
             self.install_iso or self.install_stick or self.install_pxe
@@ -1038,7 +1101,7 @@ class DiskBuilder:
     ) -> Dict:
         disk.wipe()
         disksize_used_mbytes = 0
-        if self.firmware.legacy_bios_mode():
+        if self.firmware.get_legacy_bios_partition_size():
             log.info('--> creating EFI CSM(legacy bios) partition')
             partition_mbsize = self.firmware.get_legacy_bios_partition_size()
             disk.create_efi_csm_partition(
@@ -1104,7 +1167,8 @@ class DiskBuilder:
             squashed_rootfs_mbsize = self.root_filesystem_read_only_partsize
             if not self.root_filesystem_read_only_partsize:
                 squashed_root_file = Temporary().new_file()
-                with FileSystemSquashFs(
+                with FileSystem.new(
+                    self.root_filesystem_read_only_type or 'squashfs',
                     device_provider=DeviceProvider(), root_dir=self.root_dir,
                     custom_args={
                         'compression':
@@ -1157,7 +1221,8 @@ class DiskBuilder:
                     clone_rootfs_mbsize = int(
                         (disksize_mbytes - disksize_used_mbytes) / (root_clone_count + 1)
                     ) + Defaults.get_min_partition_mbytes()
-                    rootfs_mbsize = f'clone:all_free:{clone_rootfs_mbsize}'
+                    rootfs_mbsize = \
+                        f'clone:{clone_rootfs_mbsize}:{clone_rootfs_mbsize}'
                 else:
                     rootfs_mbsize = f'clone:{rootfs_mbsize}:{rootfs_mbsize}'
             if self.volume_manager_name and self.volume_manager_name == 'lvm':
@@ -1271,6 +1336,25 @@ class DiskBuilder:
                 os.sep + os.sep.join(['etc', os.path.basename(filename)])
             )
 
+    def _write_luks_header_checksum_to_boot_image(
+        self, luks_root: Optional[LuksDevice]
+    ) -> None:
+        if luks_root is not None:
+            log.info(
+                'Including origin LUKS header checksum and key slot number'
+            )
+            filenames = [
+                ''.join([self.root_dir, '/root/.luks.header']),
+                ''.join([self.root_dir, '/root/.luks.slot']),
+                ''.join([self.root_dir, '/root/.slotpass'])
+            ]
+            for filename in filenames:
+                self.boot_image.include_file(
+                    filename=os.sep + os.sep.join(
+                        ['root', os.path.basename(filename)]
+                    ), delete_after_include=True
+                )
+
     def _write_generic_fstab_to_system_image(
         self, device_map: Dict,
         system: Optional[Union[FileSystemBase, VolumeManagerBase]]
@@ -1278,6 +1362,31 @@ class DiskBuilder:
         log.info('Creating generic system etc/fstab')
         self._write_generic_fstab(device_map, self.system_setup, system)
 
+    def _write_veritytab_to_boot_image(
+        self, device_map: Dict, veritysetup: VeritySetup
+    ) -> None:
+        log.info('Creating generic boot image etc/veritytab')
+        veritytab_filename = ''.join([self.root_dir, '/etc/veritytab'])
+        uuid = device_map['readonly'].get_uuid(
+            device_map['readonly'].get_device()
+        ) if device_map.get('readonly') else device_map['root'].get_uuid(
+            device_map['root'].get_device()
+        )
+        with open(veritytab_filename, 'w') as veritytab:
+            veritytab.write(
+                'verityroot UUID={0} UUID={0} {1} {2},{3}{4}'.format(
+                    uuid, veritysetup.verity_dict.get('Roothash'),
+                    f'hash-offset={veritysetup.verity_hash_offset}',
+                    f'hash-block-size={defaults.VERITY_HASH_BLOCKSIZE}',
+                    os.linesep
+                )
+            )
+            self.boot_image.include_file(
+                filename=os.sep + os.sep.join(
+                    ['etc', os.path.basename(veritytab_filename)]
+                ), delete_after_include=True
+            )
+
     def _write_generic_fstab_to_boot_image(
         self, device_map: Dict,
         system: Optional[Union[FileSystemBase, VolumeManagerBase]]
@@ -1292,14 +1401,14 @@ class DiskBuilder:
         self, device_map: Dict, setup: SystemSetup,
         system: Optional[Union[FileSystemBase, VolumeManagerBase]]
     ) -> None:
-        root_is_snapshot = \
-            self.xml_state.build_type.get_btrfs_root_is_snapshot()
+        root_is_snapper_snapshot = \
+            self.xml_state.build_type.get_btrfs_root_is_snapper_snapshot()
         root_is_readonly_snapshot = \
             self.xml_state.build_type.get_btrfs_root_is_readonly_snapshot()
 
         fs_check_interval = '0 1'
         custom_root_mount_args = list(self.custom_root_mount_args)
-        if root_is_snapshot and root_is_readonly_snapshot:
+        if root_is_snapper_snapshot and root_is_readonly_snapshot:
             custom_root_mount_args += ['ro']
             fs_check_interval = '0 0'
 
@@ -1451,6 +1560,10 @@ class DiskBuilder:
         if 'boot' in device_map:
             boot_device = device_map['boot']
 
+        efi_uuid = disk.get_uuid(
+            device_map['efi'].get_device()
+        ) if device_map.get('efi') else None
+
         boot_uuid = disk.get_uuid(
             boot_device.get_device()
         )
@@ -1458,7 +1571,7 @@ class DiskBuilder:
             device_map['luks_root'].get_device()
         ) if self.luks and self.boot_is_crypto else boot_uuid
         bootloader_config.setup_disk_boot_images(
-            boot_uuid_unmapped
+            boot_uuid_unmapped, efi_uuid
         )
         bootloader_config.write_meta_data(
             root_device=ro_device.
@@ -1495,7 +1608,7 @@ class DiskBuilder:
         system_spare: Optional[FileSystemBase],
         system_custom_parts: Dict[str, FileSystemBase],
         integrity_root: Optional[IntegrityDevice]
-    ) -> None:
+    ) -> Optional[Union[FileSystemBase, VolumeManagerBase]]:
         log.info('Syncing system to image')
         if system_spare:
             log.info('--> Syncing spare partition data')
@@ -1541,7 +1654,8 @@ class DiskBuilder:
         log.info('--> Syncing root filesystem data')
         if self.root_filesystem_is_overlay:
             squashed_root_file = Temporary().new_file()
-            with FileSystemSquashFs(
+            with FileSystem.new(
+                self.root_filesystem_read_only_type or 'squashfs',
                 device_provider=DeviceProvider(), root_dir=self.root_dir,
                 custom_args={
                     'compression':
@@ -1557,16 +1671,35 @@ class DiskBuilder:
                 # root tree
                 exclude_list.remove('image')
                 exclude_list.append('image/*')
-                squashed_root.create_on_file(
-                    filename=squashed_root_file.name,
-                    exclude=exclude_list
-                )
+                with MountManager(
+                    device='tmpfs', mountpoint=os.path.join(
+                        self.root_dir, 'boot'
+                    )
+                ) as overlay_boot:
+                    if system_boot:
+                        overlay_boot.tmpfs_mount()
+                        # make sure to keep boot/efi mountpoints
+                        # as they can't be created later
+                        Path.create(f'{self.root_dir}/boot/efi')
+                        for entry in [
+                            'boot/*', 'boot/.*',
+                            'boot/efi/*', 'boot/efi/.*'
+                        ]:
+                            if entry in exclude_list:
+                                exclude_list.remove(entry)
+
+                    squashed_root.create_on_file(
+                        filename=squashed_root_file.name,
+                        exclude=exclude_list
+                    )
+                    self.storage_map['system'] = squashed_root
 
                 if self.root_filesystem_verity_blocks:
                     squashed_root.create_verity_layer(
                         self.root_filesystem_verity_blocks if
                         self.root_filesystem_verity_blocks != 'all' else None
                     )
+                    self.veritysetup = squashed_root.veritysetup
 
                 readonly_target = device_map['readonly'].get_device()
                 readonly_target_bytesize = device_map['readonly'].get_byte_size(
@@ -1626,6 +1759,7 @@ class DiskBuilder:
                         label=self.disk_setup.get_root_label(),
                         uuid=BlockID(root_target).get_uuid()
                     )
+                    self.storage_map['system'] = filesystem
                     filesystem.sync_data(
                         self._get_exclude_list_for_root_data_sync(device_map)
                     )
@@ -1634,6 +1768,7 @@ class DiskBuilder:
                     self.root_filesystem_verity_blocks != 'all' else None,
                     verity_root_file.name
                 )
+                self.veritysetup = filesystem.veritysetup
 
             log.info(
                 '--> Dumping rootfs file({0} bytes) -> {1}({2} bytes)'.format(
@@ -1689,21 +1824,26 @@ class DiskBuilder:
             integrity_root.sign_integrity_metadata()
             integrity_root.write_integrity_metadata()
 
+        return self.storage_map['system']
+
     def _install_bootloader(
         self, device_map: Dict, disk,
         system: Optional[Union[FileSystemBase, VolumeManagerBase]],
-        bootloader_config: BootLoaderConfigBase
+        bootloader_config: BootLoaderConfigBase,
+        boot_image: BootImageBase
     ) -> None:
         root_device = device_map['root']
         boot_device = root_device
         readonly_device = None
+        if 'readonly' in device_map:
+            readonly_device = device_map['readonly']
+            boot_device = readonly_device
+
         if 'boot' in device_map:
             boot_device = device_map['boot']
 
-        if 'readonly' in device_map:
-            readonly_device = device_map['readonly']
-
         custom_install_arguments = {
+            'boot_image': boot_image,
             'boot_device': boot_device.get_device(),
             'root_device':
                 readonly_device.
@@ -1761,8 +1901,10 @@ class DiskBuilder:
                 bootloader.install()
             bootloader.secure_boot_install()
 
-            if self.use_disk_password and self.luks:
-                bootloader.set_disk_password(self.luks)
+            if self.use_disk_password and self.storage_map['luks_root']:
+                bootloader.set_disk_password(
+                    self.storage_map['luks_root'].passphrase
+                )
         else:
             log.warning(
                 'No install of bootcode on read-only root possible'
@@ -1772,11 +1914,11 @@ class DiskBuilder:
         self, system: Union[FileSystemBase, VolumeManagerBase]
     ) -> None:
         if self.volume_manager_name:
-            root_is_snapshot = \
-                self.xml_state.build_type.get_btrfs_root_is_snapshot()
+            root_is_snapper_snapshot = \
+                self.xml_state.build_type.get_btrfs_root_is_snapper_snapshot()
             root_is_readonly_snapshot = \
                 self.xml_state.build_type.get_btrfs_root_is_readonly_snapshot()
-            if root_is_snapshot and root_is_readonly_snapshot:
+            if root_is_snapper_snapshot and root_is_readonly_snapshot:
                 log.info(
                     'Setting root filesystem into read-only mode'
                 )
