@@ -101,9 +101,7 @@ class SystemSetup:
             pathlib.Path(f'{self.root_dir}/{defaults.LOCAL_CONTAINERS}').mkdir(
                 parents=True, exist_ok=True
             )
-            Command.run(
-                ['chroot', self.root_dir] + container.fetch_command
-            )
+            container.fetch_command(self.root_dir)
             if container.load_command:
                 container_files_to_load.append(container.container_file)
                 container_execs_to_load.append(container.load_command)
@@ -170,9 +168,20 @@ class SystemSetup:
         Command.run(
             [
                 'chroot', self.root_dir,
-                'rm', '-rf', '.kconfig', defaults.IMAGE_METADATA_DIR
+                'rm', '-f',
+                '.kconfig',
+                '.profile',
+                'config.bootoptions',
+                'config.partids'
             ]
         )
+        meta_dir = f'{self.root_dir}/{defaults.IMAGE_METADATA_DIR}'
+        if os.path.isdir(meta_dir):
+            image_meta = MountManager(
+                device='none', mountpoint=meta_dir
+            )
+            image_meta.umount()
+            Path.wipe(meta_dir)
 
     def import_repositories_marked_as_imageinclude(self) -> None:
         """
@@ -286,7 +295,8 @@ class SystemSetup:
         overlay_archive = self.description_dir + '/root.tar.gz'
         if os.path.exists(overlay_directory):
             self._sync_overlay_files(
-                overlay_directory, follow_links, preserve_owner_group
+                f'{os.path.normpath(overlay_directory)}/',
+                follow_links, preserve_owner_group
             )
         elif os.path.exists(overlay_archive):
             log.info('Extracting user defined files from archive to image tree')
@@ -357,7 +367,7 @@ class SystemSetup:
                 ['chroot', self.root_dir, 'chkstat', '--system', '--set']
             )
         else:
-            log.warning(
+            log.debug(
                 'chkstat not found in image. File Permissions Check skipped'
             )
 
@@ -554,6 +564,9 @@ class SystemSetup:
         :param str security_context_file: path file name
         """
         log.info('Processing SELinux file security contexts')
+        if not os.access(self.root_dir, os.W_OK):
+            log.info('System is read-only, security context unchanged')
+            return
         exclude = []
         for devname in Defaults.get_exclude_list_for_non_physical_devices():
             exclude.append('-e')
@@ -589,7 +602,7 @@ class SystemSetup:
             Command.run(
                 [
                     'chroot', self.root_dir, 'setfiles',
-                    '-F', '-p', '-c', self._find_selinux_policy_file(
+                    '-T0', '-F', '-p', '-c', self._find_selinux_policy_file(
                         self.xml_state.build_type.get_selinux_policy() or 'targeted'
                     )
                 ] + exclude + [
@@ -627,6 +640,25 @@ class SystemSetup:
                 options=['-a']
             )
 
+    def export_flake_pilot_system_file_list(
+        self, target_dir: str, file_name: str
+    ) -> str:
+        """
+        Export image package file list to the target_dir
+        and filename
+
+        :param str target_dir: path name
+        :param str file_name: file name
+        """
+        packager = Defaults.get_default_packager_tool(
+            self.xml_state.get_package_manager()
+        )
+        result_file = os.path.normpath(f'{target_dir}/{file_name}')
+        if packager == 'rpm':
+            self._export_rpm_flake_pilot_system_file_list(result_file)
+            return result_file
+        return ''
+
     def export_package_list(self, target_dir: str) -> str:
         """
         Export image package list as metadata reference
@@ -812,6 +844,15 @@ class SystemSetup:
             working_directory=working_directory
         )
 
+    def create_system_files(self) -> None:
+        """
+        Create file list of packages to be used by flake-pilot
+        """
+        if self.xml_state.build_type.get_provide_system_files():
+            self.export_flake_pilot_system_file_list(
+                self.root_dir, Defaults.get_system_files_name()
+            )
+
     def create_fstab(self, fstab: Fstab) -> None:
         """
         Create etc/fstab from given Fstab object
@@ -1322,6 +1363,23 @@ class SystemSetup:
         if section_content:
             return section_content[0]
 
+    def _export_rpm_flake_pilot_system_file_list(self, filename):
+        log.info('Export rpm system files script for flake-pilot')
+        dbpath_option = [
+            '--dbpath', self._get_rpm_database_location()
+        ]
+        skip_list = self.xml_state.get_system_files_ignore_packages()
+        query_call = Command.run(
+            [
+                'rpm', '--root', self.root_dir, '-qa', '--qf', '%{NAME}\n'
+            ] + dbpath_option
+        )
+        with open(filename, 'w', encoding='utf-8') as systemfiles:
+            systemfiles.write('set -e\n')
+            for package in query_call.output.splitlines():
+                if package not in skip_list:
+                    systemfiles.write(f'rpm --noghost -ql {package}\n')
+
     def _export_rpm_package_list(self, filename):
         log.info('Export rpm packages metadata')
         dbpath_option = [
