@@ -18,6 +18,7 @@ from kiwi.xml_state import XMLState
 from kiwi.xml_description import XMLDescription
 from kiwi.bootloader.config.grub2 import BootLoaderConfigGrub2
 from kiwi.utils.sysconfig import SysConfig
+from kiwi.xml_state import DracutT
 
 from kiwi.exceptions import (
     KiwiBootLoaderGrubPlatformError,
@@ -27,7 +28,8 @@ from kiwi.exceptions import (
     KiwiBootLoaderGrubDataError,
     KiwiBootLoaderGrubFontError,
     KiwiBootLoaderGrubModulesError,
-    KiwiDiskGeometryError
+    KiwiDiskGeometryError,
+    KiwiCommandNotFound
 )
 
 
@@ -61,7 +63,8 @@ class TestBootLoaderConfigGrub2:
             'root_dir/usr/lib64/efi/shim.efi': True,
             'root_dir/usr/lib64/efi/grub.efi': True,
             'root_dir/usr/lib64/efi/does-not-exist': False,
-            'root_dir/boot/efi/': True
+            'root_dir/boot/efi/': True,
+            'root_dir/usr/share/grub2/powerpc-ieee1275': True
         }
         self.glob_iglob = [
             ['root_dir/usr/lib64/efi/MokManager.efi'],
@@ -78,7 +81,7 @@ class TestBootLoaderConfigGrub2:
             return_value=None
         )
         self.firmware.efi_mode = Mock(
-            return_value=None
+            return_value=''
         )
         mock_firmware.return_value = self.firmware
 
@@ -171,9 +174,9 @@ class TestBootLoaderConfigGrub2:
     @patch('kiwi.bootloader.config.grub2.DataSync')
     @patch('kiwi.bootloader.config.grub2.Path.which')
     @patch('os.path.exists')
-    @patch('shutil.copy')
+    @patch('shutil.copy2')
     def test_setup_install_boot_images_raises_no_efigrub(
-        self, mock_shutil_copy, mock_exists, mock_Path_which,
+        self, mock_shutil_copy2, mock_exists, mock_Path_which,
         mock_sync, mock_command, mock_grub, mock_shim
     ):
         Defaults.set_platform_name('x86_64')
@@ -250,6 +253,15 @@ class TestBootLoaderConfigGrub2:
         bootloader = BootLoaderConfigGrub2(xml_state, 'root_dir')
         assert bootloader.arch == 'riscv64'
 
+    def test_post_init_loongarch64_platform(self):
+        Defaults.set_platform_name('loongarch64')
+        xml_state = MagicMock()
+        xml_state.build_type.get_firmware = Mock(
+            return_value=None
+        )
+        bootloader = BootLoaderConfigGrub2(xml_state, 'root_dir')
+        assert bootloader.arch == 'loongarch64'
+
     @patch('os.path.exists')
     def test_post_init_dom0(self, mock_exists):
         Defaults.set_platform_name('x86_64')
@@ -324,8 +336,13 @@ class TestBootLoaderConfigGrub2:
                 'some-data'
             )
 
+    @patch('kiwi.bootloader.config.grub2.DataSync.sync_data')
+    @patch('kiwi.bootloader.config.grub2.Temporary.new_dir')
     @patch('kiwi.bootloader.config.grub2.Command.run')
-    def test_create_embedded_fat_efi_image(self, mock_command):
+    def test_create_embedded_fat_efi_image(self, mock_command, mock_tmpdir, mock_syncdata):
+        tmpdir = Mock()
+        tmpdir.name = 'tmpdir'
+        mock_tmpdir.return_value = tmpdir
         self.bootloader._create_embedded_fat_efi_image('tmp-esp-image')
         assert mock_command.call_args_list == [
             call(
@@ -341,24 +358,37 @@ class TestBootLoaderConfigGrub2:
             call(
                 [
                     'mcopy', '-Do', '-s', '-i', 'tmp-esp-image',
-                    'root_dir/EFI', '::'
+                    'tmpdir', '::EFI'
                 ]
             )
         ]
 
     @patch('glob.iglob')
-    @patch('shutil.copy')
+    @patch('shutil.copy2')
     @patch('kiwi.bootloader.config.grub2.Path.create')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     @patch('kiwi.bootloader.config.grub2.Defaults.get_signed_grub_loader')
+    @patch('kiwi.bootloader.config.grub2.Path.which')
     def test_copy_grub_config_to_efi_path(
-        self, mock_get_signed_grub_loader, mock_Command,
-        mock_Path_create, mock_shutil_copy, mock_glob
+        self, mock_Path_which, mock_get_signed_grub_loader, mock_Command,
+        mock_Path_create, mock_shutil_copy2, mock_glob
     ):
         mock_get_signed_grub_loader.return_value = None
         mock_glob.return_value = []
 
-        # 1. run: check default handling
+        # 1. run: check strings command not found
+        mock_Path_which.return_value = ''
+        mock_get_signed_grub_loader.return_value = grub_loader_type(
+            'some-loader', None
+        )
+        with raises(KiwiCommandNotFound):
+            self.bootloader._copy_grub_config_to_efi_path(
+                'root_dir', 'config_file'
+            )
+
+        # 2. run: check default handling
+        mock_get_signed_grub_loader.return_value = None
+        mock_Path_which.return_value = '/usr/bin/strings'
         self.bootloader._copy_grub_config_to_efi_path(
             'root_dir', 'config_file'
         )
@@ -366,12 +396,12 @@ class TestBootLoaderConfigGrub2:
         mock_Path_create.assert_called_once_with(
             'root_dir/EFI/BOOT'
         )
-        mock_shutil_copy.assert_called_once_with(
+        mock_shutil_copy2.assert_called_once_with(
             'config_file', 'root_dir/EFI/BOOT/grub.cfg'
         )
 
-        # 2. run: check with vendor directory read from grub binary
-        mock_shutil_copy.reset_mock()
+        # 3. run: check with vendor directory read from grub binary
+        mock_shutil_copy2.reset_mock()
         mock_Path_create.reset_mock()
         strings_out = Mock()
         strings_out.output = '/EFI/ubuntu'
@@ -386,12 +416,12 @@ class TestBootLoaderConfigGrub2:
         mock_Path_create.assert_called_once_with(
             'root_dir/EFI/ubuntu'
         )
-        mock_shutil_copy.assert_called_once_with(
+        mock_shutil_copy2.assert_called_once_with(
             'config_file', 'root_dir/EFI/ubuntu/grub.cfg'
         )
 
-        # 3. run: check with installed vendor directory
-        mock_shutil_copy.reset_mock()
+        # 4. run: check with installed vendor directory
+        mock_shutil_copy2.reset_mock()
         mock_Path_create.reset_mock()
         mock_get_signed_grub_loader.return_value = None
         mock_glob.return_value = ['root_dir/EFI/fedora/shim.efi']
@@ -402,15 +432,15 @@ class TestBootLoaderConfigGrub2:
         mock_Path_create.assert_called_once_with(
             'root_dir/EFI/fedora'
         )
-        mock_shutil_copy.assert_called_once_with(
+        mock_shutil_copy2.assert_called_once_with(
             'config_file', 'root_dir/EFI/fedora/grub.cfg'
         )
 
-    @patch('shutil.copy')
+    @patch('shutil.copy2')
     @patch('os.path.exists')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     def test_setup_zipl2grub_conf_512_byte_target(
-        self, mock_Command_run, mock_exists, mock_shutil_copy
+        self, mock_Command_run, mock_exists, mock_shutil_copy2
     ):
         path_return_values = [True, False]
 
@@ -442,26 +472,26 @@ class TestBootLoaderConfigGrub2:
                 '    targetblocksize = 512\n' \
                 '    targetoffset = 2048' \
                 in file_handle.write.call_args[0][0]
-        mock_shutil_copy.assert_called_once_with(
+        mock_shutil_copy2.assert_called_once_with(
             'root_dir/etc/default/zipl2grub.conf.in',
             'root_dir/etc/default/zipl2grub.conf.in.orig'
         )
         path_return_values = [True, True]
-        mock_shutil_copy.reset_mock()
+        mock_shutil_copy2.reset_mock()
         with patch('builtins.open', create=True) as mock_open:
             file_handle = mock_open.return_value.__enter__.return_value
             file_handle.read.return_value = zipl_config
             self.bootloader._setup_zipl2grub_conf()
-        mock_shutil_copy.assert_called_once_with(
+        mock_shutil_copy2.assert_called_once_with(
             'root_dir/etc/default/zipl2grub.conf.in.orig',
             'root_dir/etc/default/zipl2grub.conf.in'
         )
 
-    @patch('shutil.copy')
+    @patch('shutil.copy2')
     @patch('os.path.exists')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     def test_setup_zipl2grub_conf_4096_byte_target(
-        self, mock_Command_run, mock_exists, mock_shutil_copy
+        self, mock_Command_run, mock_exists, mock_shutil_copy2
     ):
         path_return_values = [True, False]
         command_return_values = [
@@ -571,6 +601,63 @@ class TestBootLoaderConfigGrub2:
                 '/dev/disk', 'tracks per cylinder'
             )
 
+    @patch('os.path.exists')
+    @patch('kiwi.bootloader.config.grub2.SysConfig')
+    @patch('kiwi.bootloader.config.grub2.Command.run')
+    def test_setup_default_grub_s390_secure_execution(
+        self, mock_Command_run, mock_sysconfig, mock_exists
+    ):
+        grep_grub_option = Mock()
+        grep_grub_option.returncode = 0
+        mock_Command_run.return_value = grep_grub_option
+        mock_exists.return_value = False
+        grub_default = SysConfig('some-file')
+        grub_default.write = Mock()
+        mock_sysconfig.return_value = grub_default
+        mock_exists.return_value = True
+        self.bootloader.terminal_input = 'serial'
+        self.bootloader.terminal_output = 'gfxterm'
+        self.bootloader.theme = 'openSUSE'
+        self.bootloader.displayname = 'Bob'
+        self.bootloader.arch = 's390x'
+        self.bootloader.host_key_certificates = [
+            {
+                'hkd_cert': ['/path/to/host.crt'],
+                'hkd_revocation_list': ['some1.crl', 'some2.crl'],
+                'hkd_ca_cert': '/path/to/DigiCertCA.crt',
+                'hkd_sign_cert': '/path/to/ibm-z-host-key-signing.crt'
+            }
+        ]
+        self.firmware.efi_mode.return_value = 'efi'
+
+        self.bootloader._setup_default_grub()
+
+        mock_sysconfig.assert_called_once_with('root_dir/etc/default/grub')
+        grub_default.write.assert_called_once_with()
+
+        assert grub_default.data_dict == {
+            'GRUB_BACKGROUND': '/boot/grub2/themes/openSUSE/background.png',
+            'GRUB_CMDLINE_LINUX_DEFAULT': '"some-cmdline"',
+            'GRUB_DISTRIBUTOR': '"Bob"',
+            'GRUB_ENABLE_BLSCFG': 'true',
+            'GRUB_ENABLE_CRYPTODISK': 'y',
+            'GRUB_GFXMODE': '800x600',
+            'GRUB_SERIAL_COMMAND': '"serial --speed=38400"',
+            'GRUB_TERMINAL_INPUT': '"serial"',
+            'GRUB_TERMINAL_OUTPUT': '"gfxterm"',
+            'GRUB_THEME': '/boot/grub2/themes/openSUSE/theme.txt',
+            'GRUB_TIMEOUT': 10,
+            'GRUB_TIMEOUT_STYLE': 'countdown',
+            'SUSE_BTRFS_SNAPSHOT_BOOTING': 'true',
+            'SUSE_S390_SE_CA_CERT': '/path/to/DigiCertCA.crt',
+            'SUSE_S390_SE_ENABLE': 'true',
+            'SUSE_S390_SE_HOST_KEY': '/path/to/host.crt',
+            'SUSE_S390_SE_HOST_KEY_SIGNING_KEY':
+                '/path/to/ibm-z-host-key-signing.crt',
+            'SUSE_S390_SE_REVOCATION_LIST': 'some1.crl,some2.crl',
+            'GRUB_DEFAULT': 'saved'
+        }
+
     @patch('os.path.exists')
     @patch('kiwi.bootloader.config.grub2.SysConfig')
     @patch('kiwi.bootloader.config.grub2.Command.run')
@@ -797,6 +884,7 @@ class TestBootLoaderConfigGrub2:
         sysconfig_bootloader = MagicMock()
         mock_sysconfig.return_value = sysconfig_bootloader
         mock_exists.return_value = True
+        self.bootloader.bls = False
         self.bootloader._setup_sysconfig_bootloader()
         mock_sysconfig.assert_called_once_with(
             'root_dir/etc/sysconfig/bootloader'
@@ -837,6 +925,7 @@ class TestBootLoaderConfigGrub2:
         sysconfig_bootloader = MagicMock()
         mock_sysconfig.return_value = sysconfig_bootloader
         mock_exists.return_value = True
+        self.bootloader.bls = False
         self.bootloader._setup_sysconfig_bootloader()
         mock_sysconfig.assert_called_once_with(
             'root_dir/etc/sysconfig/bootloader'
@@ -868,6 +957,20 @@ class TestBootLoaderConfigGrub2:
             call('SECURE_BOOT', 'no'),
             call('TRUSTED_BOOT', 'yes')
         ]
+        sysconfig_bootloader.__setitem__.reset_mock()
+        self.bootloader.bls = True
+        self.bootloader._setup_sysconfig_bootloader()
+        assert sysconfig_bootloader.__setitem__.call_args_list == [
+            call('DEFAULT_APPEND', '"some-cmdline root=UUID=foo"'),
+            call(
+                'FAILSAFE_APPEND',
+                '"some-cmdline root=UUID=foo failsafe-options"'
+            ),
+            call('LOADER_LOCATION', 'none'),
+            call('LOADER_TYPE', 'grub2-bls'),
+            call('SECURE_BOOT', 'no'),
+            call('TRUSTED_BOOT', 'yes')
+        ]
 
     @patch('os.path.exists')
     def test_setup_live_image_config_custom_template(self, mock_exists):
@@ -972,13 +1075,18 @@ class TestBootLoaderConfigGrub2:
     @patch.object(BootLoaderConfigGrub2, '_copy_grub_config_to_efi_path')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     @patch('kiwi.bootloader.config.grub2.Path.which')
+    @patch('kiwi.bootloader.config.grub2.Path.create')
     @patch('kiwi.defaults.Defaults.get_vendor_grubenv')
     @patch('glob.iglob')
     def test_setup_disk_image_config(
-        self, mock_iglob, mock_get_vendor_grubenv, mock_Path_which,
+        self, mock_iglob, mock_get_vendor_grubenv,
+        mock_Path_create, mock_Path_which,
         mock_Command_run, mock_copy_grub_config_to_efi_path,
         mock_mount_system
     ):
+        self.state.get_dracut_config = Mock(
+            return_value=DracutT(uefi=False, modules=[])
+        )
         mock_iglob.return_value = ['some_entry.conf']
         mock_get_vendor_grubenv.return_value = 'grubenv'
         mock_Path_which.return_value = '/path/to/grub2-mkconfig'
@@ -1075,6 +1183,35 @@ class TestBootLoaderConfigGrub2:
             assert 'initrd /initrd' in \
                 file_handle_menu.write.call_args_list[1][0][0].split(os.linesep)
 
+            # test UKI setup
+            self.state.get_dracut_config.return_value = DracutT(
+                uefi=True, modules=[]
+            )
+            boot_image = Mock()
+            self.bootloader.setup_disk_image_config(
+                boot_options={
+                    'root_device': 'rootdev',
+                    'boot_device': 'bootdev',
+                    'boot_image': boot_image
+                }
+            )
+            boot_image.create_uki.assert_called_once_with(
+                'some-cmdline root=UUID=foo'
+            )
+
+            # test read-only device
+            file_handle_grub.write.side_effect = OSError('readonly system')
+            file_handle_grubenv.write.side_effect = OSError('readonly system')
+            file_handle_menu.write.side_effect = OSError('readonly system')
+
+            with self._caplog.at_level(logging.INFO):
+                self.bootloader.setup_disk_image_config(
+                    boot_options={
+                        'root_device': 'rootdev', 'boot_device': 'bootdev'
+                    }
+                )
+                assert 'readonly system' in self._caplog.text
+
     @patch.object(BootLoaderConfigGrub2, '_copy_grub_config_to_efi_path')
     def test_setup_install_image_config_standard(
         self, mock_copy_grub_config_to_efi_path
@@ -1356,6 +1493,23 @@ class TestBootLoaderConfigGrub2:
             )
         ]
 
+        self.state.get_dracut_config = Mock(
+            return_value=DracutT(uefi=True, modules=[])
+        )
+        with patch('builtins.open', create=True) as mock_open:
+            mock_open.return_value = MagicMock(spec=io.IOBase)
+            file_handle = mock_open.return_value.__enter__.return_value
+            self.bootloader.setup_disk_boot_images('0815', '0815')
+
+            assert file_handle.write.call_args_list == [
+                call('search --no-floppy --set=root --fs-uuid 0815\n'),
+                call('chainloader ($root)/EFI/Linux/kiwi.efi\n'),
+                call('boot\n')
+            ]
+            mock_open.assert_called_once_with(
+                'root_dir/boot/efi/EFI/BOOT/grub.cfg', 'w'
+            )
+
     @patch('kiwi.bootloader.config.base.BootLoaderConfigBase.get_boot_path')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     @patch('kiwi.bootloader.config.grub2.DataSync')
@@ -1367,7 +1521,7 @@ class TestBootLoaderConfigGrub2:
         Defaults.set_platform_name('x86_64')
         mock_get_boot_path.return_value = '/boot'
         self.firmware.efi_mode = Mock(
-            return_value=None
+            return_value=''
         )
         self.bootloader.xen_guest = True
         self.os_exists['root_dir/boot/grub2/fonts/unicode.pf2'] = False
@@ -1403,7 +1557,7 @@ class TestBootLoaderConfigGrub2:
         mock_get_boot_path.return_value = '/boot'
         self.bootloader.arch = 'ppc64le'
         self.firmware.efi_mode = Mock(
-            return_value=None
+            return_value=''
         )
         self.bootloader.xen_guest = False
         self.os_exists['root_dir/boot/grub2/fonts/unicode.pf2'] = False
@@ -1434,7 +1588,7 @@ class TestBootLoaderConfigGrub2:
         mock_get_boot_path.return_value = '/boot'
         self.bootloader.arch = 's390x'
         self.firmware.efi_mode = Mock(
-            return_value=None
+            return_value=''
         )
         self.bootloader.xen_guest = False
         self.os_exists['root_dir/boot/grub2/fonts/unicode.pf2'] = False
@@ -1485,13 +1639,6 @@ class TestBootLoaderConfigGrub2:
                         'root_dir/usr/share/grub2/i386-pc/',
                         'root_dir/boot/grub2/i386-pc'
                     ]
-                ),
-                call(
-                    [
-                        'rsync', '-a', '--exclude', '/*.module',
-                        'root_dir/usr/share/grub2/x86_64-efi/',
-                        'root_dir/boot/grub2/x86_64-efi'
-                    ]
                 )
             ]
 
@@ -1562,13 +1709,6 @@ class TestBootLoaderConfigGrub2:
                             'root_dir/boot/grub2/i386-pc'
                         ]
                     ),
-                    call(
-                        [
-                            'rsync', '-a', '--exclude', '/*.module',
-                            'root_dir/usr/share/grub2/x86_64-efi/',
-                            'root_dir/boot/grub2/x86_64-efi'
-                        ]
-                    ),
                     call(
                         [
                             'cp', 'root_dir/usr/lib64/efi/shim.efi',
@@ -1659,13 +1799,6 @@ class TestBootLoaderConfigGrub2:
                             'root_dir/boot/grub2/i386-pc'
                         ]
                     ),
-                    call(
-                        [
-                            'rsync', '-a', '--exclude', '/*.module',
-                            'root_dir/usr/share/grub2/x86_64-efi/',
-                            'root_dir/boot/grub2/x86_64-efi'
-                        ]
-                    ),
                     call(
                         [
                             'cp', 'root_dir/usr/lib64/efi/grub.efi',
@@ -1676,24 +1809,121 @@ class TestBootLoaderConfigGrub2:
 
     @patch('kiwi.bootloader.config.base.BootLoaderConfigBase.get_boot_path')
     @patch('kiwi.bootloader.config.grub2.Defaults.get_unsigned_grub_loader')
-    @patch('kiwi.bootloader.config.grub2.Defaults.get_grub_bios_core_loader')
+    @patch('kiwi.bootloader.config.grub2.Defaults.get_grub_platform_core_loader')
+    @patch('kiwi.bootloader.config.grub2.Defaults.get_grub_chrp_loader')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     @patch('kiwi.bootloader.config.grub2.Path.which')
     @patch('kiwi.bootloader.config.grub2.Path.create')
     @patch('kiwi.bootloader.config.grub2.DataSync')
     @patch('os.path.exists')
-    @patch('shutil.copy')
-    def test_setup_install_boot_images_efi(
-        self, mock_shutil_copy, mock_exists, mock_sync,
+    @patch('shutil.copy2')
+    def test_setup_install_boot_images_ppc(
+        self, mock_shutil_copy2, mock_exists, mock_sync,
         mock_Path_create, mock_Path_which, mock_command,
-        mock_get_grub_bios_core_loader, mock_get_unsigned_grub_loader,
+        mock_get_grub_chrp_loader, mock_get_grub_platform_core_loader,
+        mock_get_unsigned_grub_loader, mock_get_boot_path
+    ):
+        Defaults.set_platform_name('ppc64le')
+        mock_get_grub_chrp_loader.return_value = 'grub.elf'
+        mock_get_grub_platform_core_loader.return_value = None
+        mock_Path_which.return_value = '/path/to/grub2-mkimage'
+        mock_get_boot_path.return_value = '/boot'
+        self.bootloader.arch = 'ppc64le'
+        data = Mock()
+        mock_sync.return_value = data
+        self.firmware.efi_mode = Mock(
+            return_value=''
+        )
+
+        # simulate alternative boot directory to reach all code paths
+        self.bootloader.boot_dir = 'boot_dir'
+
+        self.os_exists['lookup_dir/usr/share/grub2/powerpc-ieee1275'] = True
+        self.os_exists['lookup_dir/usr/share/grub2/unicode.pf2'] = True
+        self.os_exists['boot_dir/boot/grub2/fonts/unicode.pf2'] = False
+        self.os_exists['boot_dir/usr/share/grub2/unicode.pf2'] = True
+
+        def side_effect(arg):
+            return self.os_exists[arg]
+
+        mock_exists.side_effect = side_effect
+
+        with open('../data/bootinfo.txt') as chrp:
+            grub2_test_chrp_boot = chrp.read()
+        with patch('builtins.open', create=True) as mock_open:
+            mock_open.return_value = MagicMock(spec=io.IOBase)
+            file_handle = mock_open.return_value.__enter__.return_value
+            self.bootloader.setup_install_boot_images(
+                mbrid=self.mbrid, lookup_path="lookup_dir"
+            )
+
+            assert mock_open.call_args_list == [
+                call('boot_dir/boot/grub2/earlyboot.cfg', 'w'),
+                call('boot_dir/boot/grub2/powerpc-ieee1275/grub.cfg', 'w'),
+                call('boot_dir/ppc/bootinfo.txt', 'w'),
+                call('boot_dir/boot/grub2/loopback.cfg', 'w')
+            ]
+
+            assert file_handle.write.call_args_list == [
+                call('set btrfs_relative_path="yes"\n'),
+                call('search --file --set=root /boot/0xffffffff\n'),
+                call('set prefix=($root)/boot/grub2\n'),
+                call('source ($root)/boot/grub2/grub.cfg\n'),
+                call('set btrfs_relative_path="yes"\n'),
+                call('search --file --set=root /boot/0xffffffff\n'),
+                call('set prefix=($root)/boot/grub2\n'),
+                call('source ($root)/boot/grub2/grub.cfg\n'),
+                call(grub2_test_chrp_boot),
+                call('source /boot/grub2/grub.cfg\n')
+            ]
+
+        assert mock_command.call_args_list == [
+            call(
+                [
+                    'cp', 'lookup_dir/usr/share/grub2/unicode.pf2',
+                    'boot_dir/boot/grub2/fonts'
+                ]
+            ),
+            call(
+                [
+                    'chroot', 'root_dir', 'grub2-mkimage',
+                    '-O', 'powerpc-ieee1275',
+                    '-o', '/usr/share/grub2/powerpc-ieee1275/grub.elf',
+                    '-c', '/boot/grub2/earlyboot.cfg',
+                    '-p', '/boot/grub2',
+                    '-d', '/usr/share/grub2/powerpc-ieee1275',
+                    'ext2', 'iso9660', 'linux', 'echo', 'configfile',
+                    'search_label', 'search_fs_file', 'search',
+                    'search_fs_uuid', 'ls', 'normal', 'gzio', 'png', 'fat',
+                    'gettext', 'font', 'minicmd', 'gfxterm', 'gfxmenu',
+                    'all_video', 'xfs', 'btrfs', 'squash4', 'lvm', 'luks',
+                    'gcry_rijndael', 'gcry_sha256', 'gcry_sha512',
+                    'crypto', 'cryptodisk', 'test', 'true', 'loadenv',
+                    'part_gpt', 'part_msdos', 'boot'
+                ]
+            )
+        ]
+
+    @patch('kiwi.bootloader.config.base.BootLoaderConfigBase.get_boot_path')
+    @patch('kiwi.bootloader.config.grub2.Defaults.get_unsigned_grub_loader')
+    @patch('kiwi.bootloader.config.grub2.Defaults.get_grub_platform_core_loader')
+    @patch('kiwi.bootloader.config.grub2.Command.run')
+    @patch('kiwi.bootloader.config.grub2.Path.which')
+    @patch('kiwi.bootloader.config.grub2.Path.create')
+    @patch('kiwi.bootloader.config.grub2.DataSync')
+    @patch('os.path.exists')
+    @patch('shutil.copy2')
+    def test_setup_install_boot_images_efi(
+        self, mock_shutil_copy2, mock_exists, mock_sync,
+        mock_Path_create, mock_Path_which, mock_command,
+        mock_get_grub_platform_core_loader, mock_get_unsigned_grub_loader,
         mock_get_boot_path
     ):
         Defaults.set_platform_name('x86_64')
         mock_Path_which.return_value = '/path/to/grub2-mkimage'
         mock_get_boot_path.return_value = '/boot'
         mock_get_unsigned_grub_loader.return_value = None
-        mock_get_grub_bios_core_loader.return_value = None
+        mock_get_grub_platform_core_loader.return_value = None
         data = Mock()
         mock_sync.return_value = data
         self.firmware.efi_mode = Mock(
@@ -1824,7 +2054,7 @@ class TestBootLoaderConfigGrub2:
             call(exclude=['*.module'], options=['-a']),
             call(exclude=['*.module'], options=['-a'])
         ]
-        assert mock_shutil_copy.call_args_list == [
+        assert mock_shutil_copy2.call_args_list == [
             call(
                 'boot_dir/boot/grub2/earlyboot.cfg',
                 'root_dir/boot/grub2/earlyboot.cfg'
@@ -1843,7 +2073,7 @@ class TestBootLoaderConfigGrub2:
             )
         ]
         mock_get_unsigned_grub_loader.return_value = 'custom_grub_image'
-        mock_get_grub_bios_core_loader.return_value = 'custom_bios_grub_image'
+        mock_get_grub_platform_core_loader.return_value = 'custom_bios_grub_image'
         mock_command.reset_mock()
 
         with patch('builtins.open', create=True) as mock_open:
@@ -1897,7 +2127,23 @@ class TestBootLoaderConfigGrub2:
                 call('source ($root)/boot/grub2/grub.cfg\n'),
             ]
 
-    @patch.object(BootLoaderConfigGrub2, '_supports_bios_modules')
+        mock_get_unsigned_grub_loader.return_value = None
+        self.state.get_dracut_config = Mock(
+            return_value=DracutT(uefi=True, modules=[])
+        )
+        with patch('builtins.open', create=True) as mock_open:
+            # Test UKI chainloader for earlyboot
+            mock_open.return_value = MagicMock(spec=io.IOBase)
+            file_handle = mock_open.return_value.__enter__.return_value
+            self.bootloader._setup_efi_image(efi_uuid='0815')
+
+            assert file_handle.write.call_args_list == [
+                call('search --no-floppy --set=root --fs-uuid 0815\n'),
+                call('chainloader ($root)/EFI/Linux/kiwi.efi\n'),
+                call('boot\n')
+            ]
+
+    @patch.object(BootLoaderConfigGrub2, '_supports_platform_modules')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     @patch('os.path.exists')
     @patch('glob.iglob')
@@ -1906,10 +2152,10 @@ class TestBootLoaderConfigGrub2:
     @patch('os.path.isfile')
     def test_setup_install_boot_images_efi_secure_boot(
         self, mock_isfile, mock_stat, mock_chmod, mock_glob,
-        mock_exists, mock_command, mock_supports_bios_modules
+        mock_exists, mock_command, mock_supports_platform_modules
     ):
         Defaults.set_platform_name('x86_64')
-        mock_supports_bios_modules.return_value = False
+        mock_supports_platform_modules.return_value = False
         mock_isfile.return_value = False
         self.os_exists['root_dir'] = True
         self.firmware.efi_mode = Mock(
@@ -1978,19 +2224,19 @@ class TestBootLoaderConfigGrub2:
                 ]
 
     @patch('kiwi.defaults.Defaults.get_grub_efi_font_directory')
-    @patch.object(BootLoaderConfigGrub2, '_supports_bios_modules')
+    @patch.object(BootLoaderConfigGrub2, '_supports_platform_modules')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     @patch('kiwi.bootloader.config.grub2.DataSync')
     @patch('os.path.exists')
     @patch('glob.iglob')
     def test_setup_install_boot_images_with_theme_from_usr_share(
         self, mock_glob, mock_exists,
-        mock_sync, mock_command, mock_supports_bios_modules,
+        mock_sync, mock_command, mock_supports_platform_modules,
         mock_get_grub_efi_font_directory
     ):
         Defaults.set_platform_name('x86_64')
         mock_get_grub_efi_font_directory.return_value = None
-        mock_supports_bios_modules.return_value = False
+        mock_supports_platform_modules.return_value = False
         mock_glob.return_value = [
             'root_dir/boot/grub2/themes/some-theme/background.png'
         ]
@@ -2094,9 +2340,9 @@ class TestBootLoaderConfigGrub2:
     @patch('kiwi.bootloader.config.grub2.DataSync')
     @patch('os.path.exists')
     @patch('kiwi.defaults.Defaults.get_grub_path')
-    @patch('shutil.copy')
+    @patch('shutil.copy2')
     def test_setup_install_boot_images_with_theme_not_existing(
-        self, mock_shutil_copy, mock_get_grub_path,
+        self, mock_shutil_copy2, mock_get_grub_path,
         mock_exists, mock_sync, mock_Path_which, mock_command
     ):
         Defaults.set_platform_name('x86_64')
