@@ -20,6 +20,7 @@ from kiwi.xml_state import XMLState
 from kiwi.builder.disk import DiskBuilder
 from kiwi.storage.disk import ptable_entry_type
 from kiwi.storage.mapped_device import MappedDevice
+from kiwi.xml_state import DracutT
 
 from kiwi.exceptions import (
     KiwiDiskBootImageError,
@@ -167,6 +168,7 @@ class TestDiskBuilder:
             return_value=self.integrity_root
         )
         self.luks_root = Mock()
+        self.luks_root.passphrase = 'passphrase'
         kiwi.builder.disk.LuksDevice = Mock(
             return_value=self.luks_root
         )
@@ -293,6 +295,7 @@ class TestDiskBuilder:
         filesystem = MagicMock()
         mock_fs.return_value = filesystem
 
+        self.firmware.get_legacy_bios_partition_size.return_value = 2
         self.disk_builder.volume_manager_name = None
         self.disk_builder.initrd_system = 'kiwi'
 
@@ -331,7 +334,7 @@ class TestDiskBuilder:
         )
         disk.map_partitions.assert_called_once_with()
         bootloader_config.setup_disk_boot_images.assert_called_once_with(
-            '0815'
+            '0815', '0815'
         )
         bootloader_config.write_meta_data.assert_called_once_with(
             root_device='/dev/readonly-root-device',
@@ -340,6 +343,7 @@ class TestDiskBuilder:
         )
         bootloader_config.setup_disk_image_config.assert_called_once_with(
             boot_options={
+                'boot_image': self.boot_image_task,
                 'boot_device': '/dev/boot-device',
                 'root_device': '/dev/readonly-root-device',
                 'write_device': '/dev/root-device',
@@ -566,7 +570,7 @@ class TestDiskBuilder:
             self.disk_setup.boot_partition_size(), 1
         )
         disk.create_root_partition.assert_called_once_with(
-            'clone:all_free:458', 1
+            'clone:458:458', 1
         )
         disk.create_custom_partitions.assert_called_once_with(
             self.disk_builder.custom_partitions
@@ -684,13 +688,18 @@ class TestDiskBuilder:
     @patch('kiwi.builder.disk.ImageSystem')
     @patch('kiwi.builder.disk.VeritySetup')
     @patch('kiwi.builder.disk.Temporary.new_file')
-    def test_create_disk_standard_root_with_dracut_initrd(
-        self, mock_Temporary_new_file, mock_VeritySetup,
+    @patch('os.access')
+    def test_create_disk_standard_root_with_dracut_initrd_readonly_root(
+        self, mock_os_access, mock_Temporary_new_file, mock_VeritySetup,
         mock_ImageSystem, mock_SystemSetup, mock_os_path_getsize,
         mock_path, mock_grub_dir, mock_command, mock_rand, mock_fs,
         mock_LoopDevice, mock_create_boot_loader_config,
         mock_Disk
     ):
+        mock_os_access.return_value = False
+        system = Mock()
+        system.mountpoint.return_value = 'diskroot'
+        mock_ImageSystem.return_value.__enter__.return_value = system
         disk = self._get_disk_instance()
         mock_Disk.return_value.__enter__.return_value = disk
         bootloader_config = Mock()
@@ -713,6 +722,89 @@ class TestDiskBuilder:
         mock_rand.return_value = 15
         filesystem = MagicMock()
         mock_fs.return_value = filesystem
+        self.firmware.get_legacy_bios_partition_size.return_value = 2
+        self.disk_builder.root_filesystem_verity_blocks = 10
+        self.disk_builder.root_filesystem_embed_verity_metadata = True
+        self.disk_builder.root_filesystem_is_overlay = False
+        self.disk_builder.volume_manager_name = None
+        self.disk_builder.initrd_system = 'dracut'
+        self.disk_builder.bootloader = 'systemd_boot'
+        self.setup.script_exists.return_value = True
+        disk_system = Mock()
+        mock_SystemSetup.return_value = disk_system
+
+        m_open = mock_open()
+        with patch('builtins.open', m_open, create=True):
+            self.disk_builder.create_disk()
+
+        assert mock_command.call_args_list == [
+            call(['cp', 'root_dir/recovery.partition.size', 'boot_dir']),
+            call(['mv', 'initrd', 'root_dir/boot/initramfs-1.2.3.img']),
+            call(['blockdev', '--getsize64', '/dev/root-device']),
+            call(['dd', 'if=tempfile', 'of=/dev/root-device']),
+            call(
+                [
+                    'blkid', '/dev/readonly-root-device',
+                    '-s', 'UUID', '-o', 'value'
+                ]
+            ),
+            call(['mv', 'initrd', 'root_dir/boot/initramfs-1.2.3.img']),
+            call(
+                [
+                    'cp', 'root_dir/boot/initramfs-1.2.3.img',
+                    'diskroot/boot/efi/os/initramfs-1.2.3.img'
+                ]
+            )
+        ]
+
+    @patch('kiwi.builder.disk.Disk')
+    @patch('kiwi.builder.disk.create_boot_loader_config')
+    @patch('kiwi.builder.disk.LoopDevice')
+    @patch('kiwi.builder.disk.FileSystem.new')
+    @patch('random.randrange')
+    @patch('kiwi.builder.disk.Command.run')
+    @patch('kiwi.builder.disk.Defaults.get_grub_boot_directory_name')
+    @patch('os.path.exists')
+    @patch('os.path.getsize')
+    @patch('kiwi.builder.disk.SystemSetup')
+    @patch('kiwi.builder.disk.ImageSystem')
+    @patch('kiwi.builder.disk.VeritySetup')
+    @patch('kiwi.builder.disk.Temporary.new_file')
+    @patch('os.access')
+    def test_create_disk_standard_root_with_dracut_initrd(
+        self, mock_os_access, mock_Temporary_new_file, mock_VeritySetup,
+        mock_ImageSystem, mock_SystemSetup, mock_os_path_getsize,
+        mock_path, mock_grub_dir, mock_command, mock_rand, mock_fs,
+        mock_LoopDevice, mock_create_boot_loader_config,
+        mock_Disk
+    ):
+        mock_os_access.return_value = True
+        system = Mock()
+        system.mountpoint.return_value = 'diskroot'
+        mock_ImageSystem.return_value.__enter__.return_value = system
+        disk = self._get_disk_instance()
+        mock_Disk.return_value.__enter__.return_value = disk
+        bootloader_config = Mock()
+        bootloader_config.get_boot_cmdline = Mock(
+            return_value='boot_cmdline'
+        )
+        mock_create_boot_loader_config.return_value.__enter__.return_value = \
+            bootloader_config
+        loop_provider = Mock()
+        mock_LoopDevice.return_value.__enter__.return_value = loop_provider
+        tempfile = Mock()
+        tempfile.name = 'tempfile'
+        mock_Temporary_new_file.return_value = tempfile
+        mock_os_path_getsize.return_value = 42
+        self.boot_image_task.get_boot_names.return_value = self.boot_names_type(
+            kernel_name='vmlinuz-1.2.3-default',
+            initrd_name='initramfs-1.2.3.img'
+        )
+        mock_path.return_value = True
+        mock_rand.return_value = 15
+        filesystem = MagicMock()
+        mock_fs.return_value = filesystem
+        self.firmware.get_legacy_bios_partition_size.return_value = 2
         self.disk_builder.root_filesystem_verity_blocks = 10
         self.disk_builder.root_filesystem_embed_verity_metadata = True
         self.disk_builder.root_filesystem_is_overlay = False
@@ -751,7 +843,7 @@ class TestDiskBuilder:
         )
         disk.map_partitions.assert_called_once_with()
         bootloader_config.setup_disk_boot_images.assert_called_once_with(
-            '0815'
+            '0815', '0815'
         )
         bootloader_config.write_meta_data.assert_called_once_with(
             root_device='/dev/readonly-root-device',
@@ -760,6 +852,7 @@ class TestDiskBuilder:
         )
         bootloader_config.setup_disk_image_config.assert_called_once_with(
             boot_options={
+                'boot_image': self.boot_image_task,
                 'boot_device': '/dev/boot-device',
                 'root_device': '/dev/readonly-root-device',
                 'write_device': '/dev/root-device',
@@ -777,6 +870,7 @@ class TestDiskBuilder:
         ]
         disk_system.call_pre_disk_script.assert_called_once_with()
         disk_system.call_disk_script.assert_called_once_with()
+        disk_system.cleanup.assert_called_once_with()
         self.setup.call_edit_boot_config_script.assert_called_once_with(
             'btrfs', 1
         )
@@ -811,20 +905,37 @@ class TestDiskBuilder:
             call('boot_dir/config.partids', 'w'),
             call('root_dir/boot/mbrid', 'w'),
             call('boot_dir/config.bootoptions', 'w'),
-            call('/dev/some-loop', 'wb')
-        ]
-        assert m_open.return_value.write.call_args_list == [
-            call('kiwi_BootPart="1"\n'),
-            call('kiwi_RootPart="1"\n'),
-            call('0x0f0f0f0f\n'),
-            call('boot_cmdline\n'),
-            call(bytes(b'\x0f\x0f\x0f\x0f'))
+            call('/dev/some-loop', 'wb'),
+            call('root_dir/etc/veritytab', 'w')
         ]
+        m_open.return_value.write.assert_has_calls(
+            [
+                call('kiwi_BootPart="1"\n'),
+                call('kiwi_RootPart="1"\n'),
+                call('0x0f0f0f0f\n'),
+                call('boot_cmdline\n'),
+                call(bytes(b'\x0f\x0f\x0f\x0f')),
+            ]
+        )
         assert mock_command.call_args_list == [
             call(['cp', 'root_dir/recovery.partition.size', 'boot_dir']),
             call(['mv', 'initrd', 'root_dir/boot/initramfs-1.2.3.img']),
             call(['blockdev', '--getsize64', '/dev/root-device']),
-            call(['dd', 'if=tempfile', 'of=/dev/root-device'])
+            call(['dd', 'if=tempfile', 'of=/dev/root-device']),
+            call(
+                [
+                    'blkid', '/dev/readonly-root-device',
+                    '-s', 'UUID', '-o', 'value'
+                ]
+            ),
+            call(['mv', 'initrd', 'root_dir/boot/initramfs-1.2.3.img']),
+            call(
+                [
+                    'cp',
+                    'root_dir/boot/initramfs-1.2.3.img',
+                    'diskroot/boot/initramfs-1.2.3.img'
+                ]
+            )
         ]
         self.block_operation.get_blkid.assert_has_calls(
             [call('PARTUUID')]
@@ -837,7 +948,8 @@ class TestDiskBuilder:
         )
         assert self.boot_image_task.include_file.call_args_list == [
             call('/config.partids'),
-            call('/recovery.partition.size')
+            call('/recovery.partition.size'),
+            call(filename='/etc/veritytab', delete_after_include=True)
         ]
         self.boot_image_task.include_module.assert_called_once_with(
             'kiwi-repart'
@@ -893,12 +1005,12 @@ class TestDiskBuilder:
             'clone:1024:1024', 1
         )
 
+    @patch('kiwi.builder.disk.Path')
     @patch('kiwi.builder.disk.Disk')
     @patch('kiwi.builder.disk.create_boot_loader_config')
     @patch('kiwi.builder.disk.LoopDevice')
     @patch('kiwi.builder.disk.DeviceProvider')
     @patch('kiwi.builder.disk.FileSystem.new')
-    @patch('kiwi.builder.disk.FileSystemSquashFs')
     @patch('kiwi.builder.disk.Command.run')
     @patch('kiwi.builder.disk.Defaults.get_grub_boot_directory_name')
     @patch('os.path.exists')
@@ -906,12 +1018,13 @@ class TestDiskBuilder:
     @patch('kiwi.builder.disk.Temporary.new_file')
     @patch('random.randrange')
     @patch('kiwi.builder.disk.BlockID')
+    @patch('kiwi.builder.disk.MountManager')
     def test_create_disk_standard_root_is_overlay(
-        self, mock_BlockID, mock_rand, mock_temp,
+        self, mock_MountManager, mock_BlockID, mock_rand, mock_temp,
         mock_getsize, mock_exists, mock_grub_dir, mock_command,
-        mock_squashfs, mock_fs, mock_DeviceProvider,
+        mock_fs, mock_DeviceProvider,
         mock_LoopDevice, mock_create_boot_loader_config,
-        mock_Disk
+        mock_Disk, mock_Path
     ):
         disk = self._get_disk_instance()
         mock_Disk.return_value.__enter__.return_value = disk
@@ -928,6 +1041,9 @@ class TestDiskBuilder:
         block_operation.get_filesystem.return_value = 'ext3'
         mock_BlockID.return_value = block_operation
         mock_rand.return_value = 15
+        self.disk_builder.dracut_setup = DracutT(uefi=True, modules=[])
+        self.disk_builder.dracut_add_modules = ['some']
+        self.disk_builder.dracut_omit_modules = ['some']
         self.disk_builder.root_filesystem_is_overlay = True
         self.disk_builder.root_filesystem_has_write_partition = False
         self.disk_builder.root_filesystem_verity_blocks = 10
@@ -935,8 +1051,12 @@ class TestDiskBuilder:
         self.disk_builder.integrity = True
         self.disk_builder.root_filesystem_embed_integrity_metadata = True
         self.disk_builder.volume_manager_name = None
-        squashfs = Mock()
-        mock_squashfs.return_value.__enter__.return_value = squashfs
+        fs = Mock()
+        fs.veritysetup.verity_hash_offset = 'Some'
+        fs.veritysetup.verity_dict = {
+            'Roothash': 'Some'
+        }
+        mock_fs.return_value.__enter__.return_value = fs
         mock_getsize.return_value = 1048576
         tempfile = Mock()
         tempfile.name = 'kiwi-tempname'
@@ -949,27 +1069,28 @@ class TestDiskBuilder:
         with patch('builtins.open', m_open, create=True):
             self.disk_builder.create_disk()
 
-        assert mock_squashfs.call_args_list == [
-            call(
-                custom_args={'compression': None},
-                device_provider=mock_DeviceProvider.return_value,
-                root_dir='root_dir'
-            ), call(
-                custom_args={'compression': None},
-                device_provider=mock_DeviceProvider.return_value,
-                root_dir='root_dir'
-            )
-        ]
-        assert squashfs.create_on_file.call_args_list == [
-            call(exclude=['var/cache/kiwi'], filename='kiwi-tempname'),
-            call(exclude=[
-                '.kconfig', 'run/*', 'tmp/*',
-                '.buildenv', 'var/cache/kiwi',
-                'boot/*', 'boot/.*', 'boot/efi/*', 'boot/efi/.*', 'image/*'
-            ], filename='kiwi-tempname')
-        ]
-        squashfs.create_verity_layer.assert_called_once_with(10)
-        squashfs.create_verification_metadata.assert_called_once_with(
+        mock_fs.assert_has_calls(
+            [
+                call(
+                    'squashfs',
+                    custom_args={'compression': None},
+                    device_provider=mock_DeviceProvider.return_value,
+                    root_dir='root_dir'
+                )
+            ]
+        )
+        fs.create_on_file.assert_has_calls(
+            [
+                call(exclude=['var/cache/kiwi'], filename='kiwi-tempname'),
+                call(exclude=[
+                    '.kconfig', 'run/*', 'tmp/*',
+                    '.buildenv', 'var/cache/kiwi',
+                    'image/*'
+                ], filename='kiwi-tempname')
+            ], any_order=True
+        )
+        fs.create_verity_layer.assert_called_once_with(10)
+        fs.create_verification_metadata.assert_called_once_with(
             '/dev/integrityRoot'
         )
         self.integrity_root.create_integrity_metadata.assert_called_once_with()
@@ -984,21 +1105,25 @@ class TestDiskBuilder:
         assert mock_command.call_args_list[3] == call(
             ['dd', 'if=kiwi-tempname', 'of=/dev/integrityRoot']
         )
-        assert m_open.return_value.write.call_args_list == [
-            # config.partids
-            call('kiwi_BootPart="1"\n'),
-            call('kiwi_RootPart="1"\n'),
-            # mbrid
-            call('0x0f0f0f0f\n'),
-            # config.bootoptions
-            call('boot_cmdline\n'),
-            # some-loop
-            call(b'\x0f\x0f\x0f\x0f')
-        ]
+        m_open.return_value.write.assert_has_calls(
+            [
+                # config.partids
+                call('kiwi_BootPart="1"\n'),
+                call('kiwi_RootPart="1"\n'),
+                # mbrid
+                call('0x0f0f0f0f\n'),
+                # config.bootoptions
+                call('boot_cmdline\n'),
+                # some-loop
+                call(b'\x0f\x0f\x0f\x0f')
+            ], any_order=True
+        )
         assert self.boot_image_task.include_module.call_args_list == [
-            call('kiwi-overlay'), call('kiwi-repart')
+            call('some'), call('kiwi-overlay'), call('kiwi-repart')
+        ]
+        assert self.boot_image_task.omit_module.call_args_list == [
+            call('some'), call('multipath')
         ]
-        self.boot_image_task.omit_module.assert_called_once_with('multipath')
         self.boot_image_task.write_system_config_file.assert_called_once_with(
             config={'modules': ['kiwi-overlay']}
         )
@@ -1107,7 +1232,7 @@ class TestDiskBuilder:
             self.disk_builder.create_disk()
 
         bootloader_config.setup_disk_boot_images.assert_called_once_with(
-            '0815'
+            '0815', '0815'
         )
 
     @patch('kiwi.builder.disk.Disk')
@@ -1177,6 +1302,7 @@ class TestDiskBuilder:
         self.disk_builder.boot_is_crypto = True
         disk.public_partition_id_map = self.id_map
         disk.public_partition_id_map['kiwi_ROPart'] = 1
+        self.disk_builder.cmdline = 'rd.kiwi.oem.luks.reencrypt'
 
         with patch('builtins.open'):
             self.disk_builder.create_disk()
@@ -1193,7 +1319,10 @@ class TestDiskBuilder:
         assert self.boot_image_task.include_file.call_args_list == [
             call('/root/.root.keyfile'),
             call('/config.partids'),
-            call('/etc/crypttab')
+            call('/etc/crypttab'),
+            call(filename='/root/.luks.header', delete_after_include=True),
+            call(filename='/root/.luks.slot', delete_after_include=True),
+            call(filename='/root/.slotpass', delete_after_include=True)
         ]
         self.boot_image_task.write_system_config_file.assert_called_once_with(
             config={'install_items': ['/root/.root.keyfile']},
@@ -1231,6 +1360,7 @@ class TestDiskBuilder:
         disk.public_partition_id_map = self.id_map
         disk.public_partition_id_map['kiwi_ROPart'] = 1
         bootloader = mock_BootLoaderInstall.return_value
+        self.disk_builder.cmdline = 'rd.kiwi.oem.luks.reencrypt'
 
         with patch('builtins.open'):
             self.disk_builder.create_disk()
@@ -1247,7 +1377,10 @@ class TestDiskBuilder:
         assert self.boot_image_task.include_file.call_args_list == [
             call('/root/.root.keyfile'),
             call('/config.partids'),
-            call('/etc/crypttab')
+            call('/etc/crypttab'),
+            call(filename='/root/.luks.header', delete_after_include=True),
+            call(filename='/root/.luks.slot', delete_after_include=True),
+            call(filename='/root/.slotpass', delete_after_include=True)
         ]
         self.boot_image_task.write_system_config_file.assert_called_once_with(
             config={'install_items': ['/root/.root.keyfile']},
