@@ -53,6 +53,9 @@ class TestSystemSetup:
         self.xml_state.get_image_version = Mock(
             return_value='1.2.3'
         )
+        self.xml_state.build_type.get_provide_system_files = Mock(
+            return_value=True
+        )
         self.xml_state.xml_data.description_dir = 'description_dir'
         self.setup = SystemSetup(
             self.xml_state, 'root_dir'
@@ -370,9 +373,24 @@ class TestSystemSetup:
     @patch('kiwi.command.Command.run')
     def test_cleanup(self, mock_command):
         self.setup.cleanup()
-        mock_command.assert_called_once_with(
-            ['chroot', 'root_dir', 'rm', '-rf', '.kconfig', 'image']
-        )
+        assert mock_command.call_args_list == [
+            call(
+                [
+                    'chroot', 'root_dir', 'rm', '-f',
+                    '.kconfig',
+                    '.profile',
+                    'config.bootoptions',
+                    'config.partids'
+                ]
+            ),
+            call(
+                command=['mountpoint', '-q', 'root_dir/image'],
+                raise_on_error=False
+            ),
+            call(
+                ['rm', '-r', '-f', 'root_dir/image']
+            )
+        ]
 
     @patch('kiwi.system.setup.ArchiveTar')
     @patch('kiwi.system.setup.glob.iglob')
@@ -1282,6 +1300,14 @@ class TestSystemSetup:
     ):
         assert self.setup.export_package_list('target_dir') == ''
 
+    @patch('kiwi.defaults.Defaults.get_default_packager_tool')
+    def test_export_flake_pilot_system_file_list_unknown_packager(
+        self, mock_get_default_packager_tool
+    ):
+        assert self.setup.export_flake_pilot_system_file_list(
+            'target_dir', 'system_files'
+        ) == ''
+
     @patch('kiwi.defaults.Defaults.get_default_packager_tool')
     def test_export_package_changes_unknown_packager(
         self, mock_get_default_packager_tool
@@ -1294,6 +1320,42 @@ class TestSystemSetup:
     ):
         assert self.setup.export_package_verification('target_dir') == ''
 
+    @patch('kiwi.system.setup.Command.run')
+    @patch('kiwi.system.setup.RpmDataBase')
+    @patch('kiwi.system.setup.MountManager')
+    def test_export_flake_pilot_system_file_list(
+        self, mock_MountManager, mock_RpmDataBase, mock_command_run
+    ):
+        rpmdb = Mock()
+        rpmdb.rpmdb_image.expand_query.return_value = 'image_dbpath'
+        rpmdb.rpmdb_host.expand_query.return_value = 'host_dbpath'
+        rpmdb.has_rpm.return_value = True
+        mock_RpmDataBase.return_value = rpmdb
+        self.xml_state.get_system_files_ignore_packages = Mock(
+            return_value=['rpm', 'zypper']
+        )
+        mock_command_run.return_value = Mock(output='glibc\nzypper\n')
+
+        with patch('builtins.open') as mock_open:
+            mock_open.return_value = MagicMock(spec=io.IOBase)
+            file_handle = mock_open.return_value.__enter__.return_value
+            result = self.setup.export_flake_pilot_system_file_list(
+                'target_dir', 'system_files'
+            )
+            mock_open.assert_called_once_with(
+                'target_dir/system_files', 'w', encoding='utf-8'
+            )
+            assert file_handle.write.call_args_list == [
+                call('set -e\n'), call('rpm --noghost -ql glibc\n')
+            ]
+        assert result == 'target_dir/system_files'
+        mock_command_run.assert_called_once_with(
+            [
+                'rpm', '--root', 'root_dir',
+                '-qa', '--qf', '%{NAME}\n', '--dbpath', 'image_dbpath'
+            ]
+        )
+
     @patch('kiwi.system.setup.Command.run')
     @patch('kiwi.system.setup.RpmDataBase')
     @patch('kiwi.system.setup.MountManager')
@@ -1392,6 +1454,15 @@ class TestSystemSetup:
         with self._caplog.at_level(logging.WARNING):
             self.setup.setup_permissions()
 
+    @patch.object(SystemSetup, 'export_flake_pilot_system_file_list')
+    def test_create_system_files(
+        self, mock_export_flake_pilot_system_file_list
+    ):
+        self.setup.create_system_files()
+        mock_export_flake_pilot_system_file_list.assert_called_once_with(
+            'root_dir', 'systemfiles'
+        )
+
     @patch('kiwi.system.setup.Command.run')
     def test_export_package_list_dpkg(self, mock_command):
         command = Mock()
@@ -1596,13 +1667,27 @@ class TestSystemSetup:
         mock_command.assert_called_once_with(
             [
                 'chroot', 'root_dir',
-                'setfiles', '-F', '-p',
+                'setfiles', '-T0', '-F', '-p',
                 '-c', '/etc/selinux/targeted/policy/policy.some_policy',
                 '-e', '/proc', '-e', '/sys', '-e', '/dev',
                 'security_context_file', '/'
             ]
         )
 
+    @patch('kiwi.system.setup.CommandCapabilities.has_option_in_help')
+    @patch('kiwi.system.setup.Command.run')
+    @patch('os.scandir')
+    @patch('os.access')
+    def test_set_selinux_file_contexts_read_only_root(
+        self, mock_os_access, mock_os_scandir, mock_command,
+        mock_has_option_in_help
+    ):
+        mock_os_access.return_value = False
+        mock_has_option_in_help.return_value = False
+        mock_os_scandir.return_value = self.selinux_policies
+        self.setup.set_selinux_file_contexts('security_context_file')
+        assert not mock_command.called
+
     @patch('kiwi.system.setup.CommandCapabilities.has_option_in_help')
     @patch('kiwi.system.setup.Command.run')
     @patch('os.scandir')
