@@ -10,9 +10,7 @@ from pytest import (
 )
 
 from kiwi.defaults import Defaults
-from kiwi.xml_state import (
-    XMLState, ContainerT
-)
+from kiwi.xml_state import XMLState
 from kiwi.storage.disk import ptable_entry_type
 from kiwi.xml_description import XMLDescription
 
@@ -154,6 +152,12 @@ class TestXMLState:
         assert self.state.get_bootstrap_packages() == [
             'dnf', 'filesystem',
         ]
+        self.state.get_package_manager = Mock(
+            return_value="apk"
+        )
+        assert self.state.get_bootstrap_packages() == [
+            'apk-tools', 'filesystem',
+        ]
 
     def test_get_system_packages(self):
         assert self.state.get_system_packages() == [
@@ -252,7 +256,7 @@ class TestXMLState:
     def test_set_repository(self):
         self.state.set_repository(
             'repo', 'type', 'alias', 1, True, False, ['key_a', 'key_b'],
-            'main universe', 'jammy', False
+            'main universe', 'jammy', False, 'metalink'
         )
         assert self.state.xml_data.get_repository()[0].get_source().get_path() \
             == 'repo'
@@ -273,11 +277,13 @@ class TestXMLState:
             == 'jammy'
         assert self.state.xml_data.get_repository()[0] \
             .get_repository_gpgcheck() is False
+        assert self.state.xml_data.get_repository()[0] \
+            .get_sourcetype() == 'metalink'
 
     def test_add_repository(self):
         self.state.add_repository(
             'repo', 'type', 'alias', 1, True, None, ['key_a', 'key_b'],
-            'main universe', 'jammy', False
+            'main universe', 'jammy', False, 'metalink'
         )
         assert self.state.xml_data.get_repository()[3].get_source().get_path() \
             == 'repo'
@@ -296,6 +302,8 @@ class TestXMLState:
             == 'jammy'
         assert self.state.xml_data.get_repository()[3] \
             .get_repository_gpgcheck() is False
+        assert self.state.xml_data.get_repository()[3] \
+            .get_sourcetype() == 'metalink'
 
     def test_add_repository_with_empty_values(self):
         self.state.add_repository('repo', 'type', '', '', True)
@@ -306,12 +314,19 @@ class TestXMLState:
         assert self.state.xml_data.get_repository()[3].get_priority() is None
         assert self.state.xml_data.get_repository()[3] \
             .get_imageinclude() is True
+        assert self.state.xml_data.get_repository()[3] \
+            .get_sourcetype() is None
 
     def test_get_to_become_deleted_packages(self):
         assert self.state.get_to_become_deleted_packages() == [
             'kernel-debug'
         ]
 
+    def test_get_system_files_ignore_packages(self):
+        assert self.state.get_system_files_ignore_packages() == [
+            'rpm', 'yast', 'zypp'
+        ]
+
     def test_get_build_type_vagrant_config_section(self):
         vagrant_config = self.state.get_build_type_vagrant_config_section()
         assert vagrant_config.get_provider() == 'libvirt'
@@ -412,55 +427,104 @@ class TestXMLState:
             )
         }
 
-    def test_get_containers(self):
-        assert self.state.get_containers() == [
-            ContainerT(
-                name='rmtserver_latest',
-                backend='podman',
-                container_file='/var/tmp/kiwi_containers/rmtserver_latest',
-                fetch_only=False,
-                fetch_command=[
-                    '/usr/bin/skopeo', 'copy',
-                    'docker://registry.suse.com/home/mschaefer/'
-                    'images_pubcloud/pct/rmtserver:latest',
-                    'oci-archive:/var/tmp/kiwi_containers/'
-                    'rmtserver_latest:registry.suse.com/home/mschaefer/'
-                    'images_pubcloud/pct/rmtserver:latest'
-                ],
-                load_command=[
-                    '/usr/bin/podman', 'load', '-i',
-                    '/var/tmp/kiwi_containers/rmtserver_latest'
-                ]
-            ),
-            ContainerT(
-                name='some_latest',
-                backend='docker',
-                container_file='/var/tmp/kiwi_containers/some_latest',
-                fetch_only=False,
-                fetch_command=[
-                    '/usr/bin/skopeo', 'copy',
-                    'docker://registry.suse.com/some:latest',
-                    'oci-archive:/var/tmp/kiwi_containers/'
-                    'some_latest:registry.suse.com/some:latest'
-                ],
-                load_command=[
-                    '/usr/bin/docker', 'load', '-i',
-                    '/var/tmp/kiwi_containers/some_latest'
-                ]
-            ),
-            ContainerT(
-                name='foo_latest',
-                backend='podman',
-                container_file='/var/tmp/kiwi_containers/foo_latest',
-                fetch_only=True,
-                fetch_command=[
-                    '/usr/bin/skopeo', 'copy', 'docker://docker.io/foo:latest',
-                    'oci-archive:/var/tmp/kiwi_containers/'
-                    'foo_latest:docker.io/foo:latest'
-                ],
-                load_command=[]
-            )
+    @patch('kiwi.xml_state.Defaults.is_buildservice_worker')
+    @patch('kiwi.xml_state.Command.run')
+    def test_get_containers_in_buildservice(
+        self, mock_Command_run, mock_Defaults_is_buildservice_worker
+    ):
+        mock_Defaults_is_buildservice_worker.return_value = True
+        description = XMLDescription(
+            '../data/example_containers_config.xml'
+        )
+        xml_data = description.load()
+        state = XMLState(xml_data)
+        containers = state.get_containers()[0]
+        containers.fetch_command('root_dir')
+        assert containers.name == 'tumbleweed_latest'
+        assert containers.backend == 'podman'
+        assert containers.container_file == \
+            '/var/tmp/kiwi_containers/tumbleweed_latest'
+        assert containers.fetch_only is False
+        assert containers.load_command == [
+            '/usr/bin/podman', 'load', '-i',
+            '/var/tmp/kiwi_containers/tumbleweed_latest'
         ]
+        mock_Command_run.assert_called_once_with(
+            [
+                'cp',
+                '/usr/src/packages/SOURCES/containers/'
+                '_obsrepositories/registry.opensuse.org/opensuse/'
+                'tumbleweed:latest.ociarchive',
+                'root_dir/var/tmp/kiwi_containers/tumbleweed_latest'
+            ]
+        )
+
+    @patch('kiwi.xml_state.Command.run')
+    def test_get_containers(self, mock_Command_run):
+        containers = self.state.get_containers()
+        c1 = containers[0]
+        c2 = containers[1]
+        c3 = containers[2]
+
+        c1.fetch_command('root_dir')
+        assert c1.name == 'rmtserver_latest'
+        assert c1.backend == 'podman'
+        assert c1.container_file == \
+            '/var/tmp/kiwi_containers/rmtserver_latest'
+        assert c1.fetch_only is False
+        assert c1.load_command == [
+            '/usr/bin/podman', 'load', '-i',
+            '/var/tmp/kiwi_containers/rmtserver_latest'
+        ]
+        mock_Command_run.assert_called_once_with(
+            [
+                'chroot', 'root_dir',
+                '/usr/bin/skopeo', 'copy',
+                'docker://registry.suse.com/home/mschaefer/'
+                'images_pubcloud/pct/rmtserver:latest',
+                'oci-archive:/var/tmp/kiwi_containers/'
+                'rmtserver_latest:registry.suse.com/home/mschaefer/'
+                'images_pubcloud/pct/rmtserver:latest'
+            ]
+        )
+        mock_Command_run.reset_mock()
+
+        c2.fetch_command('root_dir')
+        assert c2.name == 'some_latest'
+        assert c2.backend == 'docker'
+        assert c2.container_file == \
+            '/var/tmp/kiwi_containers/some_latest'
+        assert c2.fetch_only is False
+        assert c2.load_command == [
+            '/usr/bin/docker', 'load', '-i',
+            '/var/tmp/kiwi_containers/some_latest'
+        ]
+        mock_Command_run.assert_called_once_with(
+            [
+                'chroot', 'root_dir',
+                '/usr/bin/skopeo', 'copy',
+                'docker://registry.suse.com/some:latest',
+                'oci-archive:/var/tmp/kiwi_containers/'
+                'some_latest:registry.suse.com/some:latest'
+            ]
+        )
+        mock_Command_run.reset_mock()
+
+        c3.fetch_command('root_dir')
+        assert c3.name == 'foo_latest'
+        assert c3.backend == 'podman'
+        assert c3.container_file == \
+            '/var/tmp/kiwi_containers/foo_latest'
+        assert c3.fetch_only is True
+        assert c3.load_command == []
+        mock_Command_run.assert_called_once_with(
+            [
+                'chroot', 'root_dir',
+                '/usr/bin/skopeo', 'copy', 'docker://docker.io/foo:latest',
+                'oci-archive:/var/tmp/kiwi_containers/'
+                'foo_latest:docker.io/foo:latest'
+            ]
+        )
 
     def test_get_volumes_custom_root_volume_name(self):
         description = XMLDescription(
@@ -1058,14 +1122,14 @@ class TestXMLState:
     def test_get_derived_from_image_uri(self):
         xml_data = self.description.load()
         state = XMLState(xml_data, ['derivedContainer'], 'docker')
-        assert state.get_derived_from_image_uri().uri == \
+        assert state.get_derived_from_image_uri()[0].uri == \
             'obs://project/repo/image#mytag'
 
     def test_set_derived_from_image_uri(self):
         xml_data = self.description.load()
         state = XMLState(xml_data, ['derivedContainer'], 'docker')
         state.set_derived_from_image_uri('file:///new_uri')
-        assert state.get_derived_from_image_uri().translate() == '/new_uri'
+        assert state.get_derived_from_image_uri()[0].translate() == '/new_uri'
 
     def test_set_derived_from_image_uri_not_applied(self):
         with self._caplog.at_level(logging.WARNING):
@@ -1144,7 +1208,12 @@ class TestXMLState:
     @patch('kiwi.xml_parse.type_.get_bootloader')
     def test_get_build_type_bootloader_bls(self, mock_bootloader):
         mock_bootloader.return_value = [self.bootloader]
+        self.bootloader.get_bls.return_value = False
         assert self.state.get_build_type_bootloader_bls() is False
+        self.bootloader.get_bls.return_value = True
+        assert self.state.get_build_type_bootloader_bls() is True
+        self.bootloader.get_bls.return_value = None
+        assert self.state.get_build_type_bootloader_bls() is True
 
     @patch('kiwi.xml_parse.type_.get_bootloader')
     def test_get_build_type_bootloader_console(self, mock_bootloader):
@@ -1183,12 +1252,20 @@ class TestXMLState:
             'some-target'
 
     def test_get_installintrd_modules(self):
-        self.state.get_installmedia_initrd_modules('add') == ['network-legacy']
-        self.state.get_installmedia_initrd_modules('set') == []
-        self.state.get_installmedia_initrd_modules('omit') == []
+        assert self.state.get_installmedia_initrd_modules('add') == \
+            ['network-legacy']
+        assert self.state.get_installmedia_initrd_modules('set') == []
+        assert self.state.get_installmedia_initrd_modules('omit') == []
         xml_data = self.description.load()
         state = XMLState(xml_data, ['vmxSimpleFlavour'], 'oem')
-        state.get_installmedia_initrd_modules('add') == []
+        assert state.get_installmedia_initrd_modules('add') == []
+
+    def test_get_dracut_config(self):
+        assert self.state.get_dracut_config('setup').uefi is False
+        xml_data = self.description.load()
+        state = XMLState(xml_data, ['vmxSimpleFlavour'], 'oem')
+        assert state.get_dracut_config('setup').uefi is True
+        assert state.get_dracut_config('add').modules == ['some']
 
     @patch('kiwi.system.uri.os.path.abspath')
     def test_get_repositories_signing_keys(self, mock_root_path):
@@ -1254,6 +1331,25 @@ class TestXMLState:
             '--joe', '-x'
         ]
 
+    def test_get_host_key_certificates(self):
+        description = XMLDescription('../data/example_hkd_config.xml')
+        xml_data = description.load()
+        state = XMLState(xml_data)
+        assert state.get_host_key_certificates() == [
+            {
+                'hkd_cert': ['some1-host.crt', 'some2-host.crt'],
+                'hkd_revocation_list': ['some1-revocation.crl'],
+                'hkd_ca_cert': 'some-ca.crt',
+                'hkd_sign_cert': 'some1-signing.crt'
+            },
+            {
+                'hkd_cert': ['some3-host.crt'],
+                'hkd_revocation_list': ['some2-revocation.crl'],
+                'hkd_ca_cert': 'some-ca.crt',
+                'hkd_sign_cert': 'some2-signing.crt'
+            }
+        ]
+
     def test_get_btrfs_root_is_subvolume(self):
         assert self.state.build_type.get_btrfs_root_is_subvolume() is \
             None
